# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import time
import numpy as np

from pybpodapi.com.arcom import ArduinoTypes
from pybpodapi.bpod_modules.bpod_modules import BpodModules
from pybpodapi.bpod.bpod_com_protocol import BpodCOMProtocol
from pybpodapi.com.protocol.send_msg_headers import SendMessageHeader
from pybpodapi.com.protocol.recv_msg_headers import ReceiveMessageHeader
from pybpodapi.exceptions.bpod_error import BpodErrorException


logger = logging.getLogger(__name__)


class BpodCOMProtocolModules(BpodCOMProtocol):
    """
    Define command actions that can be requested to Bpod device.

    **Private attributes**

        _arcom
            :class:`pybpodapi.com.arcom.ArCOM`

            ArCOM object that performs serial communication.

    **Methods**

    """

    def _bpodcom_get_modules_info(self, hardware):

        bpod_modules = BpodModules(self)  # type: BpodModules

        input_modules = [inp for inp in hardware.inputs if inp == "U"]
        n_modules = len(input_modules)
        n_serial_events = int(hardware.max_serial_events / (n_modules + 1))

        self._arcom.write_char(SendMessageHeader.GET_MODULES)
        time.sleep(0.3)

        modules_requested_events = np.array([0] * n_modules)

        names = {}

        if self._arcom.bytes_available() > 1:
            for i in range(n_modules):
                connected = self._arcom.read_uint8() == 1
                firmware_version = None
                module_name = None
                events_names = []

                if connected:
                    firmware_version = self._arcom.read_uint32()
                    name_length = self._arcom.read_uint8()
                    name_string = self._arcom.read_char_array(name_length)
                    name_string = "".join(name_string)

                    if name_string not in names:
                        names[name_string] = 0
                    names[name_string] += 1

                    module_name = name_string + str(names[name_string])

                    flag = self._arcom.read_uint8()
                    while flag == 1:  # has more info to be read
                        param_type = self._arcom.read_uint8()

                        if param_type == ReceiveMessageHeader.MODULE_REQUESTED_EVENT:
                            modules_requested_events[i] = self._arcom.read_uint8()

                        elif param_type == ReceiveMessageHeader.MODULE_EVENT_NAMES:

                            n_event_names = self._arcom.read_uint8()
                            for _ in range(n_event_names):
                                n_chars = self._arcom.read_uint8()
                                event_name = self._arcom.read_char_array(n_chars)
                                events_names.append("".join(event_name))

                        flag = self._arcom.read_uint8()

                bpod_modules += BpodModules.create_module(
                    connected,
                    module_name,
                    firmware_version,
                    events_names,
                    n_serial_events,
                    serial_port=i + 1,
                )

        if (
            modules_requested_events.sum() + n_serial_events
        ) > hardware.max_serial_events:
            raise BpodErrorException(
                "Error: Connected modules requested too many events."
            )

        for i, module in enumerate(bpod_modules):
            if module.connected:

                if modules_requested_events[i] > module.n_serial_events:
                    n_to_reassign = modules_requested_events[i] - module.n_serial_events
                    module.n_serial_events = modules_requested_events[i]
                else:
                    n_to_reassign = 0

                index = n_modules - 1
                while n_to_reassign > 0:
                    if bpod_modules[index].n_serial_events >= n_to_reassign:
                        bpod_modules[index].n_serial_events = (
                            bpod_modules[index].n_serial_events - n_to_reassign
                        )
                        n_to_reassign = 0
                    else:
                        n_to_reassign = (
                            n_to_reassign - bpod_modules[index].n_serial_events
                        )
                        bpod_modules[index].n_serial_events = 0
                    index -= 1

        n_serial_events_array = [m.n_serial_events for m in bpod_modules]
        n_soft_codes = hardware.max_serial_events - sum(n_serial_events_array)

        bytes2send = ArduinoTypes.get_uint8_array(
            [ord("%")] + n_serial_events_array + [n_soft_codes]
        )

        self._arcom.write_array(bytes2send)
        res = self._arcom.read_uint8()

        if not res:
            raise BpodErrorException(
                "Error: Failed to configure module event assignment."
            )

        return bpod_modules

    def _bpodcom_activate_module_relay(self, module_index):
        self._arcom.write_array(
            [ord(SendMessageHeader.SET_MODULE_RELAY), module_index, 1]
        )

    def _bpodcom_deactivate_module_relay(self, module_index):
        self._arcom.write_array(
            [ord(SendMessageHeader.SET_MODULE_RELAY), module_index, 0]
        )

    def _bpodcom_clean_any_data_in_the_buffer(self):
        n_bytes_available = self.bytes_available()
        if n_bytes_available > 0:
            self._arcom.read_uint8_array(n_bytes_available)

    def _bpodcom_module_write(self, module_index, message, dtype=None):
        if dtype is None:
            dtype = ArduinoTypes.UINT8

        if isinstance(message, str):
            message = [ord(c) for c in message]
        elif not isinstance(message, (list, np.ndarray)):
            message = [message]
        else:
            message = message

        msg = ArduinoTypes.get_array(message, dtype)

        if len(msg) > 64:
            raise BpodErrorException(
                "Error: module messages must be under 64 bytes per transmission"
            )

        to_send = [ord(SendMessageHeader.WRITE_TO_MODULE), module_index + 1, len(msg)]
        to_send = ArduinoTypes.get_uint8_array(to_send)

        self._arcom.write_array(to_send + msg)

    def _bpodcom_module_read(self, module_index, size, dtype=None):
        if dtype is None:
            dtype = ArduinoTypes.UINT8

        if dtype == ArduinoTypes.CHAR:
            return self._arcom.read_char_array(size)
        elif dtype == ArduinoTypes.UINT8:
            return self._arcom.read_uint8_array(size)
        elif dtype == ArduinoTypes.UINT16:
            return self._arcom.read_uint16_array(size)
        elif dtype == ArduinoTypes.UINT32:
            return self._arcom.read_uint32_array(size)
        elif dtype == ArduinoTypes.FLOAT32:
            return self._arcom.read_float32_array(size)
        else:
            return None



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import logging
import pybpodgui_api
from pybpodgui_api.models.board.board_base import BoardBase
from sca.formats import json


logger = logging.getLogger(__name__)


class BoardIO(BoardBase):
    """

    """

    def __init__(self, project):
        super(BoardIO, self).__init__(project)

        self.data = None

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def collect_data(self, data):
        data.update({'name': self.name})
        data.update({'serial_port': self.serial_port})

        return data

    def save(self):
        """
        Save experiment data on filesystem.

        :ivar dict parent_path: Project path.
        :return: Dictionary containing the board info to save. If None is returned, it means that ther was a failure.
        :rtype: dict
        """
        if not self.name:
            logger.warning("Skipping board without name")
            return None
        else:
            if not os.path.exists(self.path):
                os.makedirs(self.path)

            if self.data:
                data = self.data
            else:
                data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url='http://pybpod.readthedocs.org',
                    def_text='This file contains the configuration of Bpod board.'
                )
            data['serial-port'] = self.serial_port
            data['enabled-bncports'] = self.enabled_bncports
            data['enabled-wiredports'] = self.enabled_wiredports
            data['enabled-behaviorports'] = self.enabled_behaviorports
            data['net-port'] = self.net_port

            config_path = os.path.join(self.path, self.name+'.json')
            with open(config_path, 'w') as fstream:
                json.dump(data, fstream)

    def load(self, path):
        """
        Load board data from filesystem

        :ivar str board_path: Path of the board
        :ivar dict data: data object that contains all board info
        """
        self.name = os.path.basename(path)
        with open(os.path.join(self.path, self.name+'.json'), 'r') as stream:
            self.data = data = json.load(stream)

        self.uuid4 = data.uuid4 if data.uuid4 else self.uuid4
        self.serial_port = data.get('serial-port', data.get('serial_port', None))
        self.enabled_bncports = data.get('enabled-bncports', None)
        self.enabled_wiredports = data.get('enabled-wiredports', None)
        self.enabled_behaviorports = data.get('enabled-behaviorports', None)
        self.net_port = data.get('net-port', None)

    def __generate_boards_path(self, project_path):
        return os.path.join(project_path, 'boards')

    def __generate_board_path(self, boards_path):
        return os.path.join(boards_path, self.name)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import math
from pybpodapi.com.arcom import ArduinoTypes

from pybpodapi.com.protocol.send_msg_headers import SendMessageHeader
from pybpodapi.state_machine.state_machine_base import StateMachineBase

logger = logging.getLogger(__name__)


class StateMachineBuilder(StateMachineBase):
    """
    Extend state machine with builder logic

    .. warning:: A lot of data structures are kept here for compatibility with original matlab library which are not so python-like. Anyone is welcome to enhance this class but keep in mind that it will affect the whole pybpodapi library.
    """

    def update_state_numbers(self):
        """
        Replace undeclared states (at the time they were referenced) with actual state numbers
        """
        for i in range(len(self.undeclared)):
            undeclaredStateNumber = i + 10000
            thisStateNumber = self.manifest.index(self.undeclared[i])
            for j in range(self.total_states_added):
                if self.state_timer_matrix[j] == undeclaredStateNumber:
                    self.state_timer_matrix[j] = thisStateNumber

                # input matrix
                inputTransitions = self.input_matrix[j]
                for k in range(0, len(inputTransitions)):
                    thisTransition = inputTransitions[k]
                    if thisTransition[1] == undeclaredStateNumber:
                        inputTransitions[k] = (thisTransition[0], thisStateNumber)
                self.input_matrix[j] = inputTransitions

                # start matrix
                inputTransitions = self.global_timers.start_matrix[j]
                for k in range(0, len(inputTransitions)):
                    thisTransition = inputTransitions[k]
                    if thisTransition[1] == undeclaredStateNumber:
                        inputTransitions[k] = (thisTransition[0], thisStateNumber)
                self.global_timers.start_matrix[j] = inputTransitions

                # end matrix
                inputTransitions = self.global_timers.end_matrix[j]
                for k in range(0, len(inputTransitions)):
                    thisTransition = inputTransitions[k]
                    if thisTransition[1] == undeclaredStateNumber:
                        inputTransitions[k] = (thisTransition[0], thisStateNumber)
                self.global_timers.end_matrix[j] = inputTransitions

                # global counters
                inputTransitions = self.global_counters.matrix[j]
                for k in range(0, len(inputTransitions)):
                    thisTransition = inputTransitions[k]
                    if thisTransition[1] == undeclaredStateNumber:
                        inputTransitions[k] = (thisTransition[0], thisStateNumber)
                self.global_counters.matrix[j] = inputTransitions

                # conditions
                inputTransitions = self.conditions.matrix[j]
                for k in range(0, len(inputTransitions)):
                    thisTransition = inputTransitions[k]
                    if thisTransition[1] == undeclaredStateNumber:
                        inputTransitions[k] = (thisTransition[0], thisStateNumber)
                self.conditions.matrix[j] = inputTransitions

        # Check to make sure all states in manifest exist
        logger.debug(
            "Total states added: %s | Manifested sates: %s",
            self.total_states_added,
            len(self.manifest),
        )
        if len(self.manifest) > self.total_states_added:
            raise StateMachineBuilderError(
                "Error: some states were referenced by name, but not subsequently declared."
            )

    def build_header(self, run_asap=None, statemachine_body_size=0):
        message = [ord(SendMessageHeader.NEW_STATE_MATRIX)]
        message += [0 if run_asap is None else 1]
        message += [1 if self.use_255_back_signal else 0]

        return ArduinoTypes.get_uint8_array(message) + ArduinoTypes.get_uint16_array(
            [statemachine_body_size]
        )

    def build_message(self):
        """
        Builds state machine to send to Bpod box

        :rtype: list(int)
        """
        self.highest_used_global_counter = self.global_counters.get_max_index_used()
        self.highest_used_global_timer = self.global_timers.get_max_index_used()
        self.highest_used_global_condition = self.conditions.get_max_index_used()

        self.highest_used_global_counter = (
            0
            if self.highest_used_global_counter is None
            else self.highest_used_global_counter + 1
        )
        self.highest_used_global_timer = (
            0
            if self.highest_used_global_timer is None
            else self.highest_used_global_timer + 1
        )
        self.highest_used_global_condition = (
            0
            if self.highest_used_global_condition is None
            else self.highest_used_global_condition + 1
        )

        message = [
            self.total_states_added,
            self.highest_used_global_timer,
            self.highest_used_global_counter,
            self.highest_used_global_condition,
        ]

        # STATE TIMER MATRIX
        # Send state timer transitions (for all states)
        tmp = []
        for i in range(self.total_states_added):
            tmp += [
                self.total_states_added
                if math.isnan(self.state_timer_matrix[i])
                else self.state_timer_matrix[i]
            ]

        message += tmp
        logger.debug("STATE TIMER MATRIX: %s", tmp)

        # INPUT MATRIX
        # Send event-triggered transitions (where they are different from default)
        tmp = []
        for i in range(self.total_states_added):

            state_transitions = self.input_matrix[i]
            n_transitions = len(state_transitions)
            tmp += [n_transitions]
            for transition in state_transitions:
                tmp += [transition[0]]
                dest_state = transition[1]
                tmp += [
                    self.total_states_added if math.isnan(dest_state) else dest_state
                ]
        message += tmp
        logger.debug("INPUT MATRIX: %s", tmp)

        # OUTPUT MATRIX
        # Send hardware states (where they are different from default)
        tmp = []
        for i in range(self.total_states_added):
            hw_state = self.output_matrix[i]
            hw_state = [
                evt
                for evt in hw_state
                if evt[0] < self.hardware.channels.events_positions.globalTimerTrigger
            ]
            n_differences = len(hw_state)
            tmp += [n_differences]
            for hw_conf in hw_state:
                tmp += hw_conf[:2]
        message += tmp
        logger.debug("OUTPUT MATRIX: %s", tmp)

        # GLOBAL_TIMER_START_MATRIX
        # Send global timer-start triggered transitions (where they are different from default)
        tmp = []
        for i in range(self.total_states_added):
            state_transitions = self.global_timers.start_matrix[i]
            n_transitions = len(state_transitions)
            tmp += [n_transitions]
            for transition in state_transitions:
                dest_state = transition[1]
                tmp += [
                    transition[0]
                    - self.hardware.channels.events_positions.globalTimerStart
                ]
                tmp += [
                    self.total_states_added if math.isnan(dest_state) else dest_state
                ]
        message += tmp
        logger.debug("GLOBAL_TIMER_START_MATRIX: %s", tmp)

        # GLOBAL_TIMER_END_MATRIX
        # Send global timer-end triggered transitions (where they are different from default)
        tmp = []
        for i in range(self.total_states_added):
            state_transitions = self.global_timers.end_matrix[i]
            n_transitions = len(state_transitions)
            tmp += [n_transitions]
            for transition in state_transitions:
                dest_state = transition[1]
                tmp += [
                    transition[0]
                    - self.hardware.channels.events_positions.globalTimerEnd
                ]
                tmp += [
                    self.total_states_added if math.isnan(dest_state) else dest_state
                ]
        message += tmp
        logger.debug("GLOBAL_TIMER_END_MATRIX: %s", tmp)

        # GLOBAL_COUNTER_MATRIX
        # Send global counter triggered transitions (where they are different from default)
        tmp = []
        for i in range(self.total_states_added):
            state_transitions = self.global_counters.matrix[i]
            n_transitions = len(state_transitions)
            tmp += [n_transitions]
            for transition in state_transitions:
                dest_state = transition[1]
                tmp += [
                    transition[0]
                    - self.hardware.channels.events_positions.globalCounter
                ]
                tmp += [
                    self.total_states_added if math.isnan(dest_state) else dest_state
                ]
        message += tmp
        logger.debug("GLOBAL_COUNTER_MATRIX: %s", tmp)

        # CONDITION_MATRIX
        # Send condition triggered transitions (where they are different from default)
        tmp = []
        for i in range(self.total_states_added):
            state_transitions = self.conditions.matrix[i]
            n_transitions = len(state_transitions)
            tmp += [n_transitions]
            for transition in state_transitions:
                dest_state = transition[1]
                tmp += [
                    transition[0] - self.hardware.channels.events_positions.condition
                ]
                tmp += [
                    self.total_states_added if math.isnan(dest_state) else dest_state
                ]

        message += tmp
        logger.debug("CONDITION_MATRIX: %s", tmp)

        # GLOBAL_TIMER_CHANNELS
        tmp = []
        for i in range(self.highest_used_global_timer):
            tmp += [self.global_timers.channels[i]]
        message += tmp
        logger.debug("GLOBAL_TIMER_CHANNELS: %s", tmp)

        # GLOBAL_TIMER_ON_MESSAGES
        tmp = []
        for i in range(self.highest_used_global_timer):
            v = self.global_timers.on_messages[i]
            tmp += [255 if v == 0 else v]
        message += tmp
        logger.debug("GLOBAL_TIMER_ON_MESSAGES: %s", tmp)

        # GLOBAL_TIMER_OFF_MESSAGES
        tmp = []
        for i in range(self.highest_used_global_timer):
            v = self.global_timers.off_messages[i]
            tmp += [255 if v == 0 else v]
        message += tmp
        logger.debug("GLOBAL_TIMER_OFF_MESSAGES: %s", tmp)

        # GLOBAL_TIMER_LOOP_MODE
        tmp = []
        for i in range(self.highest_used_global_timer):
            tmp += [self.global_timers.loop_mode[i]]
        message += tmp
        logger.debug("GLOBAL_TIMER_LOOP_MODE: %s", tmp)

        # GLOBAL_TIMER_EVENTS
        tmp = []
        for i in range(self.highest_used_global_timer):
            tmp += [self.global_timers.send_events[i]]
        message += tmp
        logger.debug("GLOBAL_TIMER_EVENTS: %s", tmp)

        # GLOBAL_COUNTER_ATTACHED_EVENTS
        tmp = []
        for i in range(self.highest_used_global_counter):
            tmp += [self.global_counters.attached_events[i]]
        message += tmp
        logger.debug("GLOBAL_COUNTER_ATTACHED_EVENTS: %s", tmp)

        # CONDITIONS_CHANNELS
        tmp = []
        for i in range(self.highest_used_global_condition):
            tmp += [self.conditions.channels[i]]
        message += tmp
        logger.debug("CONDITIONS_CHANNELS: %s", tmp)

        # CONDITIONS VALUES
        tmp = []
        for i in range(self.highest_used_global_condition):
            tmp += [self.conditions.values[i]]
        message += tmp
        logger.debug("CONDITIONS VALUES: %s", tmp)

        # GLOBAL_COUNTER_RESETS
        tmp = []
        for i in range(self.total_states_added):
            tmp += [self.global_counters.reset_matrix[i]]
        message += tmp
        logger.debug("GLOBAL_COUNTER_RESETS: %s", tmp)

        self.state_timers = self.state_timers[: self.total_states_added]

        return ArduinoTypes.get_uint8_array(message)

    def build_message_global_timer(self):

        message = []

        for i in range(self.total_states_added):
            message += (self.global_timers.triggers_matrix[i],)

        for i in range(self.total_states_added):
            message += (self.global_timers.cancels_matrix[i],)

        for i in range(self.highest_used_global_timer):
            message += (self.global_timers.onset_matrix[i],)

        if self.hardware.n_global_timers > 16:
            return ArduinoTypes.get_uint32_array(message)
        elif self.hardware.n_global_timers > 8:
            return ArduinoTypes.get_uint16_array(message)
        else:
            return ArduinoTypes.get_uint8_array(message)

    def build_message_32_bits(self):
        """
        Builds a 32 bit message to send to Bpod box

        :rtype: list(float)
        """
        # TODO find how many global timers are used

        """
        thirty_two_bit_message = [i * self.hardware.cycle_frequency for i in self.state_timers] + \
                                 [i * self.hardware.cycle_frequency for i in self.global_timers.timers] + \
                                 [i * self.hardware.cycle_frequency for i in self.global_timers.on_set_delays] + \
                                 [i * self.hardware.cycle_frequency for i in self.global_timers.loop_intervals] + \
                                 self.global_counters.thresholds"""

        used_timers = range(self.highest_used_global_timer)
        used_counters = range(self.highest_used_global_counter)

        thirty_two_bit_message = (
            [t * self.hardware.cycle_frequency for t in self.state_timers]
            + [
                self.global_timers.timers[i] * self.hardware.cycle_frequency
                for i in used_timers
            ]
            + [
                self.global_timers.on_set_delays[i] * self.hardware.cycle_frequency
                for i in used_timers
            ]
            + [
                self.global_timers.loop_intervals[i] * self.hardware.cycle_frequency
                for i in used_timers
            ]
            + [self.global_counters.thresholds[i] for i in used_counters]
        )

        return ArduinoTypes.get_uint32_array(thirty_two_bit_message)


class StateMachineBuilderError(Exception):
    pass



# !/usr/bin/python3
# -*- coding: utf-8 -*-


import logging
import os
import importlib
import sys

from confapp import conf

logger = logging.getLogger(__name__)


def install_plugins(main_window):
	""" Load plugins from plugins folder. These plugins should
	inherit from BaseWidget """

	logger.info("------------- LOADING PLUGINS -------------")

	plugins_path = conf.GENERIC_EDITOR_PLUGINS_PATH
	plugins_list = conf.GENERIC_EDITOR_PLUGINS_LIST

	# plugins_2_load = []

	if plugins_path:
		if os.path.exists(plugins_path):
			for filename in os.listdir(plugins_path):
				if not filename.startswith('.'):
					if filename.endswith('.py') \
							or filename.endswith('.zip') \
							or os.path.isdir(os.path.realpath(os.path.join(plugins_path, filename))):
						plugin_folder = os.path.realpath(os.path.join(plugins_path, filename))
						sys.path.insert(0, os.path.realpath(plugin_folder))
						logger.info("Added user plugin: %s", plugin_folder)
						logger.debug("Sys path now: %s", sys.path)
		else:
			logger.warning("Plugins path does not exists")

	else:
		logger.warning("Plugins path was not defined by user")

	# plugins_2_load = plugins_2_load + plugins_list

	for plugin_name in plugins_list:
		logger.info("Installing plugin: {0}".format(plugin_name))
		try:
			m = importlib.import_module(plugin_name)
			conf.GENERIC_EDITOR_PLUGINS_FINDER += plugin_name
			main_window.plugins_classes.append(m)
		except Exception as err:
			logger.warning("Skiping bad plugin: %s", err, exc_info=True)

	logger.info("------------- END LOADING PLUGINS -------------")



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pybpodgui_plugin.models.session.session_treenode import SessionTreeNode

logger = logging.getLogger(__name__)


class SessionDockWindow(SessionTreeNode):

    def show(self):
        self.mainwindow.details.value = self
        super(SessionDockWindow, self).show()

    @property
    def mainwindow(self):
        return self.setup.mainwindow

    def beforeClose(self):
        return False



# !/usr/bin/python3
# -*- coding: utf-8 -*-
from pybpodapi.com.messaging.base_message import BaseMessage


class ErrorMessage(BaseMessage):
    """
    Stderr message from the server process

    .. seealso::

        :py:class:`pybpodgui_plugin.com.messaging.board_message.BoardMessage`

    """

    MESSAGE_TYPE_ALIAS = "error"
    MESSAGE_COLOR = (240, 0, 0)



from AnyQt import QtCore
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText, ControlButton, ControlLabel
from pybpod_alyx_module.module_api import AlyxModule
from AnyQt.QtWidgets import QLineEdit

from pybpodgui_api.models.project import Project
from pybpod_alyx_module.models.subject.alyx_subject import AlyxSubject
from pybpodgui_api.models.subject import Subject
from pybpodgui_api.models.user import User

class UserWindow(User, AlyxModule, BaseWidget):

    TITLE = 'Alyx connection'

    def __init__(self, _project):
        BaseWidget.__init__(self, self.TITLE)
        AlyxModule.__init__(self)
        User.__init__(self, _project)

        self.project = _project
        
        self._namebox = ControlText('User:')
        self._password = ControlText('Password:')
        self._connect_btn = ControlButton('Connect',default = self._connect)
        self._status_lbl = ControlLabel('Status: Not Connected')
        self._getsubjects_btn = ControlButton('Get Subjects', default = self._get_subjects)

        self.set_margin(10)

        self._namebox.value = self._name

        self._namebox.changed_event = self.__name_changed_evt
        
        self._password.form.lineEdit.setEchoMode(QLineEdit.Password)

        self.formset = [
            '_namebox',
            '_password',
            '_connect_btn',
            '_status_lbl',
            '_getsubjects_btn'
        ]

        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)

    def _connect(self):
        if self._connect_to_alyx(self._namebox.value,self._password.value):
            self._status_lbl.value = 'Status: CONNECTED'

    def _get_subjects(self):
        result = self.get_alyx_subjects(self._name)
        for subj in result:
            subjname = subj['nickname']
            existing = False
            for s in self.project.subjects:
                if s.name == subjname:
                    existing = True
                    reply = self.question('Subject' + s.name + 'Already exists locally. Replace details?', 'Update Subject')
                    if reply == 'yes':
                        s.add_alyx_info(subj)
            if existing == False:
                newsubject = AlyxSubject(self.project)
                newsubject.add_alyx_info(subj)

    
    def __name_changed_evt(self):
        if not hasattr(self, '_update_name') or not self._update_name:
            self.name = self._namebox.value
            

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._update_name = True  # Flag to avoid recursive calls when editing the name text field
        self._name = value
        self._update_name = False


# !/usr/bin/python3
# -*- coding: utf-8 -*-
from pybpodapi.com.messaging.base_message import BaseMessage
from pybpodapi.utils import date_parser


class SessionInfo(BaseMessage):
    """
    Stderr message from the server process

    .. seealso::

        :py:class:`pybpodgui_plugin.com.messaging.board_message.BoardMessage`

    """

    MESSAGE_TYPE_ALIAS = "INFO"
    MESSAGE_COLOR = (150, 150, 255)

    def __init__(self, infoname, infovalue=None, start_time=None, end_time=None):
        super(SessionInfo, self).__init__(infoname, host_timestamp=start_time)
        self._infovalue = infovalue
        self._endtime = end_time

    def tolist(self):
        return [
            self.MESSAGE_TYPE_ALIAS,
            self.pc_timestamp,
            self.host_timestamp,
            self._endtime,
            self.content,
            self._infovalue,
        ]

    @classmethod
    def fromlist(cls, row):
        """
        Returns True if the typestr represents the class
        """
        obj = cls(row[4], float(row[2]) if row[2] else None)
        obj.pc_timestamp = date_parser.parse(row[1])
        obj._infovalue = row[5] if len(row) > 5 else None
        return obj

    @property
    def infoname(self):
        return self.content

    @property
    def infovalue(self):
        return self._infovalue



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import subprocess
import os


class TaskCommand(object):

    WHEN_PRE = 0
    WHEN_POST = 1

    def __init__(self, task=None):
        self.task = task
        self.task += self

        self.when = None

    def execute(self):
        pass


class ScriptCmd(TaskCommand):

    def __init__(self, task=None):
        super(ScriptCmd, self).__init__(task)
        self.script = ''

    def execute(self, **kwargs):
        global_dict = globals()
        global_dict.update(kwargs)
        script_path = os.path.join(self.task.path, self.script)

        environment = os.environ.copy()
        environment['PYTHONPATH'] = os.pathsep.join([os.path.abspath(self.task.path)])

        proc = subprocess.Popen(
            ['python', os.path.abspath(script_path)],
            cwd=self.task.path,
            env=environment
        )
        proc.wait()
        # exec(open(script_path).read(), global_dict)

    def __str__(self):
        return self.script

    def save(self):
        """
        Save board task data on filesystem.

        :ivar str setup_path: Setup path.
        :return: Dictionary containing the board task info to save.
        :rtype: dict
        """
        return {'type': 'create_scriptcmd', 'when': self.when, 'script': self.script}

    def load(self, data):
        """
        Load setup data from filesystem

        :ivar str setup_path: Path of the setup
        :ivar dict data: data object that contains all setup info
        """
        self.when = data['when']
        self.script = data['script']

    def collect_data(self, data):
        data.update({'script': self.script})
        data.update({'when': self.when})

        return data


class ExecCmd(TaskCommand):

    def __init__(self, task=None):
        super(ExecCmd, self).__init__(task)
        self.cmd = ''

    def execute(self, **kwargs):
        p = subprocess.Popen(
            self.cmd.split(' '),
            cwd=self.task.path,
            # stdin=subprocess.PIPE,
            # stdout=subprocess.PIPE,
            # stderr=subprocess.PIPE
        )

    def __str__(self):
        return self.cmd

    def save(self):
        """
        Save board task data on filesystem.

        :ivar str setup_path: Setup path.
        :return: Dictionary containing the board task info to save.
        :rtype: dict
        """
        return {'type': 'create_execcmd', 'when': self.when, 'cmd': self.cmd}

    def load(self, data):
        """
        Load setup data from filesystem

        :ivar str setup_path: Path of the setup
        :ivar dict data: data object that contains all setup info
        """
        self.when = data['when']
        self.cmd = data['cmd']

    def collect_data(self, data):
        data.update({'cmd': self.cmd})
        data.update({'when': self.when})

        return data



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from AnyQt.QtGui import QIcon

from pyforms.controls import ControlTree

from pybpodgui_plugin_stmdiagram.stmdiagram_window import StmDiagramWindow

logger = logging.getLogger(__name__)


class SessionTreeNode(object):

    def create_treenode(self, tree):
        """

        :param ControlTree tree: project tree

        :return: this session tree node
        """
        node = super().create_treenode(tree)
        self.stmdiagram_action = tree.add_popup_menu_option(
            'STM Diagram',
            self.__open_stmdiagram_plugin,
            item=self.node,
            icon=QIcon(conf.TIMELINE_PLUGIN_ICON)
        )
        return node

    def __open_stmdiagram_plugin(self):

        self.load_contents()

        if not hasattr(self, 'stmdiagram_win'):
            self.stmdiagram_win = StmDiagramWindow(self)
            self.stmdiagram_win.show()
            self.stmdiagram_win.subwindow.resize(*conf.TIMELINE_PLUGIN_WINDOW_SIZE)
        else:
            self.stmdiagram_win.show()

        self.stmdiagram_action.setEnabled(False)

    def remove(self):
        if hasattr(self, 'stmdiagram_win'):
            self.mainwindow.mdi_area -= self.stmdiagram_win
        super().remove()

    @property
    def name(self):
        return super(SessionTreeNode, self.__class__).name.fget(self)

    @name.setter
    def name(self, value):
        super(SessionTreeNode, self.__class__).name.fset(self, value)
        if hasattr(self, 'stmdiagram_win'):
            self.stmdiagram_win.title = value



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf
from AnyQt.QtGui import QIcon
from pybpodgui_plugin.models.board import Board
from pybpodgui_plugin.models.project.project_dockwindow import ProjectDockWindow

logger = logging.getLogger(__name__)


class ProjectUIBusy(ProjectDockWindow):
    """

    """

    def update_ui(self):
        """
        Update user interface
        """
        busy_status = Board.STATUS_READY

        # This boolean makes it easy to disable run buttons on subjects
        sessionrunning = False

        logger.debug('Project [{0}] status:{1}'.format(self.name, busy_status))

        for board in self.boards:
            if board.status > Board.STATUS_READY:
                busy_status = board.status
                break

        if busy_status == Board.STATUS_READY:

            self.node.setIcon(0, QIcon(conf.PROJECT_SMALL_ICON))
            self.experiments_node.setIcon(0, QIcon(conf.EXPERIMENTS_SMALL_ICON))
            self.boards_node.setIcon(0, QIcon(conf.BOARDS_SMALL_ICON))
            self.subjects_node.setIcon(0, QIcon(conf.SUBJECTS_SMALL_ICON))

        elif busy_status in [Board.STATUS_RUNNING_TASK]:

            self.node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            self.experiments_node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            self.boards_node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            self.subjects_node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            # Flag this true so we can disable 'Run' buttons
            sessionrunning = True

        for exp in self.experiments:
            exp.update_ui()

        for board in self.boards:
            board.update_ui()

        for subj in self.subjects:
            subj.update_ui(sessionrunning)



import logging
from confapp import conf

from AnyQt.QtGui import QIcon
from AnyQt import QtCore

from pybpodgui_plugin.models.user.user_window import UserWindow

logger = logging.getLogger(__name__)


class UserTreeNode(UserWindow):
    def __init__(self, _project):
        UserWindow.__init__(self, _project)
        self.project = _project
        self.create_treenode(self.tree)

    def create_treenode(self, tree):
        self.node = tree.create_child(self._name, self.project.users_node, icon=QIcon(conf.PERSON_SMALL_ICON))
        self.node.key_pressed_event = self.node_key_pressed_event
        # print(self.node)
        self.node.window = self
        self.node.double_clicked_event = self.node_double_clicked_event

        tree.add_popup_menu_option('Remove', self.remove, item=self.node, icon=QIcon(conf.REMOVE_SMALL_ICON))
        return self.node

    def node_double_clicked_event(self):
        self.connection = 'local'
        self.project.loggeduser = self

    def remove(self):
        self.project.user_removed(self)
        self.project -= self
        self.project.users_node.removeChild(self.node)

    def node_key_pressed_event(self, event):
        """
        Sets key events for:
            * Remove board: :meth:`BoardTreeNode.remove`

        :param event: key event
        """
        if event.key() == QtCore.Qt.Key_Delete:
            self.remove()

    @property
    def name(self):
        if hasattr(self, 'node'):
            return str(self.node.text(0))
        else:
            return UserWindow.name.fget(self)

    @name.setter
    def name(self, value):
        UserWindow.name.fset(self, value)
        if hasattr(self, 'node'):
            self.node.setText(0, value)

    @property
    def tree(self):
        return self.project.tree



from pybpod_gui_plugin_emulator.models.setup.setup_window_emulator import SetupWindowEmulator


class SetupUIBusyEmulator(SetupWindowEmulator):

    def update_ui(self):
        super(SetupUIBusyEmulator, self).update_ui()

        # update button on self._emulator_gui
        if hasattr(self, 'emulator_plugin'):
            if self.status == self.STATUS_READY:
                self.emulator_plugin._run_task_btn.checked = False
                self.emulator_plugin._run_task_btn.label = 'Run protocol'
                self.emulator_plugin._kill_task_btn.enabled = False
                self.emulator_plugin._stop_trial_btn.enabled = False
                self.emulator_plugin._pause_btn.label = 'Pause'
                self.emulator_plugin._pause_btn.enabled = False
                self.emulator_plugin._pause_btn.checked = False

            elif self.status == self.STATUS_BOARD_LOCKED:
                print("board locked")
            elif self.status == self.STATUS_RUNNING_TASK:
                self.emulator_plugin._run_task_btn.checked = True
                self.emulator_plugin._run_task_btn.label = 'Stop'
                self.emulator_plugin._kill_task_btn.enabled = True
                self.emulator_plugin._stop_trial_btn.enabled = True
                self.emulator_plugin._pause_btn.enabled = True



SUBJECTS = '/subjects'
USERS = '/users'
AUTH_TOKEN = '/auth-token'
HEADERS = {
    'Authorization': 'Token 4aab2b946ee6b55ab94162584e5bec702a481e1f',
    'Accept': 'application/json',
    'Content-Type': 'application/json',
}


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class ReceiveMessageHeader(object):
    """
    Define names for message headers received from the Bpod device.

    The message header is the first byte (character) on a message received.
    """

    #: Success code from HANDSHAKE command
    HANDSHAKE_OK = "5"

    #: Success code from ENABLE_PORTS command
    ENABLE_PORTS_OK = 1

    #: Success code from SYNC_CHANNEL_MODE command
    SYNC_CHANNEL_MODE_OK = 1

    #: Success code from RUN_STATE_MACHINE command
    STATE_MACHINE_INSTALLATION_STATUS = 1

    #: Success code from LOAD_SERIAL_MESSAGE command
    LOAD_SERIAL_MESSAGE_OK = 1

    #: Success code from RESET_SERIAL_MESSAGES command
    RESET_SERIAL_MESSAGES = 1

    #: Module requested event
    MODULE_REQUESTED_EVENT = ord("#")

    #: Module events names
    MODULE_EVENT_NAMES = ord("E")



import logging
from pybpodapi.com.messaging.error import ErrorMessage
from pybpodapi.com.messaging.debug import DebugMessage
from pybpodapi.com.messaging.stderr import StderrMessage
from pybpodapi.com.messaging.stdout import StdoutMessage
from pybpodapi.com.messaging.warning import WarningMessage

logger = logging.getLogger(__name__)


class MessageParser(object):

    COLUMN_SEPARATOR = ";"

    MESSAGES_TYPES_CLASSES = [
        ErrorMessage,
        DebugMessage,
        StderrMessage,
        StdoutMessage,
        WarningMessage,
    ]

    def fromlist(self, row):
        """
        Parses messages saved on session history file

        .. seealso::

            :py:meth:`pybpodgui_plugin.api.models.session.session_io.SessionIO.load_contents`


        :param str txtline: file line entry
        :returns: list of history messages
        :rtype: list(BaseMessage)
        """
        if row is None or len(row) == 0:
            return ErrorMessage("Parse error: line is empty")

        msg = None
        try:
            msgtype = row[0]

            for msgtype_class in self.MESSAGES_TYPES_CLASSES:
                if msgtype_class.check_type(msgtype):
                    msg = msgtype_class.fromlist(row)
                    break
        except Exception:
            logger.warning(
                "Could not parse bpod message: {0}".format(str(row)), exc_info=True
            )
            return ErrorMessage(row)  # default case

        return msg



import requests
import json
from confapp import conf

class Post():

    def __init__(self,_apibase):
        self.apibase = _apibase

    def registerfile(self, _path, _dns, _created_by, _filenames, _projects):
        _data = dict(path = _path, dns = _dns, created_by = _created_by, filenames = _filenames, projects = _projects)
        
        #requests.post(conf.ALYX_PLUGIN_ADDRESS + '/register-file',headers = self.apibase.headers, data = _data)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pyforms.controls import ControlTree

from pyforms_generic_editor.models.projects.projects_window import ProjectsWindow

logger = logging.getLogger(__name__)


class ProjectsTreeNode(ProjectsWindow):
	def __init__(self, mainwindow=None):
		ProjectsWindow.__init__(self, mainwindow)

		self.tree = ControlTree('Projects')

		self.tree.item_selection_changed_event = self.__item_sel_changed_evt
		self.clear()

	def __item_sel_changed_evt(self):
		"""
		Show window from selected node (if exists)
		"""
		try:
			selected = self.tree.selected_item
			if hasattr(selected, 'window'):
				selected.window.show()
		except Exception as err:  # CATCH UNEXPECTED ERRORS FROM WINDOWS PREVENTING APP KILL
			logger.error(str(err), exc_info=True)

	def clear(self):
		self.tree.setDragEnabled(False)
		self.tree.show_header = False
		# self.tree.setHeaderLabel('Please open or create project')
		self.tree.clear()

	def create_project(self):
		"""
		Invoke project creation and focus GUI on the new tree node.

		.. seealso::
			* Create project: :class:`pybehavior.models.projects.projects_window.ProjectsWindow.create_project`.

		:rtype: Project
		"""
		pass

	def open_project(self, project_path=None):
		"""
		Open project. We want to temporarily silent the item_selection_changed_event becasue otherwise, when loading
		the project, entities will show up on the MDI area (and not in the dockwindow).
		Finally, we have to force the project node to change.

		.. seealso::
			* Open project: :class:`pybehavior.models.projects.projects_window.ProjectsWindow.open_project`.

		:param str project_path:
		"""
		pass



import time
import importlib
from confapp import conf


class BpodModules(object):

    LOADED_MODULES = []

    def __init__(self, bpod):
        self.modules = []
        self.bpod = bpod

    def __add__(self, module):
        module.bpod_modules = self
        self.modules.append(module)
        return self

    def __getitem__(self, index):
        return self.modules[index]

    def __len__(self):
        return len(self.modules)

    def __iter__(self):
        return iter(self.modules)

    @staticmethod
    def create_module(
        connected,
        module_name,
        firmware_version,
        events_names,
        n_serial_events,
        serial_port,
    ):
        from pybpodapi.bpod_modules.bpod_module import (
            BpodModule,
        )  # solve issue related with circular imports

        if len(BpodModules.LOADED_MODULES) == 0 and len(conf.PYBPOD_API_MODULES) > 0:

            for module2import in conf.PYBPOD_API_MODULES:
                m = importlib.import_module(module2import)
                BpodModules.LOADED_MODULES.append(m.BpodModule)

            for mclass in BpodModules.LOADED_MODULES:
                if mclass.check_module_type(module_name):
                    return mclass(
                        connected,
                        module_name,
                        firmware_version,
                        events_names,
                        n_serial_events,
                        serial_port,
                    )

        return BpodModule(
            connected,
            module_name,
            firmware_version,
            events_names,
            n_serial_events,
            serial_port,
        )

    def activate_module_relay(self, module):
        index = self.modules.index(module)
        self.bpod._bpodcom_activate_module_relay(index)

    def deactivate_module_relay(self, module):
        index = self.modules.index(module)
        self.bpod._bpodcom_deactivate_module_relay(index)

    def stop_modules_relay(self):
        for m in self.modules:
            m.deactivate_module_relay()

        time.sleep(0.1)
        self.bpod._bpodcom_clean_any_data_in_the_buffer()

    def module_write(self, module, message, dtype=None):
        index = self.modules.index(module)
        self.bpod._bpodcom_module_write(index, message, dtype)

    def module_read(self, module, size, dtype=None):
        index = self.modules.index(module)
        return self.bpod._bpodcom_module_read(index, size, dtype)

    @property
    def relay_is_active(self):
        for m in self.modules:
            if m.relay_active:
                return True

        return False



import io
import sys

from pybpodapi.com.messaging.stdout import StdoutMessage


class StdoutBuffer(io.StringIO):
    def __init__(self, session):
        super().__init__()
        self.session = session

    def write(self, msg):
        self.session += StdoutMessage(msg)

    def flush(self):
        sys.__stdout__.flush()



# !/usr/bin/python3
# -*- coding: utf-8 -*-

"""
A board represents the hardware that controls the running session for a specific setup.
"""

import os
import uuid
import logging

from pybpodgui_api.utils.generate_name import generate_name

logger = logging.getLogger(__name__)


class BoardBase(object):
    """
    Board base class with main attributes. A board should have a name, serial port, project belonging to, a path and a
    list of messages.
    """

    def __init__(self, project):
        """
        :ivar Project project: Project to which the Board belongs to.
        """
        self.uuid4 = uuid.uuid4()

        self.name = generate_name([x.name for x in project.boards], "box")
        self.serial_port = None
        self.project = project
        self.net_port = 36000 + len(project.boards)

        self.data = None
        self.log_messages = []

        self.project += self

        self.enabled_bncports = None
        self.enabled_wiredports = None
        self.enabled_behaviorports = None

    def __add__(self, value):
        self.log_messages.append(value)
        return self

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################
    @property
    def name(self):
        """
        Get and set the board name

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def serial_port(self):
        """
        Get and set the board serial port

        :rtype: str
        """
        return self._serial_port

    @serial_port.setter
    def serial_port(self, serial_port):
        self._serial_port = serial_port

    @property
    def project(self):
        """
        Get and set the board project

        :rtype: str
        """
        return self._project

    @project.setter
    def project(self, value):
        self._project = value

    @property
    def path(self):
        """
        Get and set the board path

        :rtype: str
        """
        if self.project.path is None:
            return None
        return os.path.join(self.project.path, 'boards', self.name)

    @property
    def enabled_bncports(self):
        """
        Get and set enabled bncports

        :rtype: list(Boolean)
        """
        return self._enabled_bncports

    @enabled_bncports.setter
    def enabled_bncports(self, value):
        self._enabled_bncports = value

    @property
    def enabled_wiredports(self):
        """
        Get and set the enabled wired ports

        :rtype: list(Boolean)
        """
        return self._enabled_wiredports

    @enabled_wiredports.setter
    def enabled_wiredports(self, value):
        self._enabled_wiredports = value

    @property
    def enabled_behaviorports(self):
        """
        Get and set the experiment name

        :rtype: list(Boolean)
        """
        return self._enabled_behaviorports

    @enabled_behaviorports.setter
    def enabled_behaviorports(self, value):
        self._enabled_behaviorports = value

    @property
    def net_port(self):
        """
        Get and set the experiment name

        :rtype: list(Boolean)
        """
        return self._net_port

    @net_port.setter
    def net_port(self, value):
        self._net_port = value

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def remove(self):
        pass

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.__unicode__()



# !/usr/bin/python3
# -*- coding: utf-8 -*-
from pybpodapi.com.messaging.base_message import BaseMessage
from pybpodapi.utils import date_parser


class ValueMessage(BaseMessage):
    """
    Stderr message from the server process

    .. seealso::

        :py:class:`pybpodgui_plugin.com.messaging.board_message.BoardMessage`

    """

    MESSAGE_TYPE_ALIAS = "VAL"
    MESSAGE_COLOR = (240, 0, 0)

    def __init__(self, value_name, value, host_timestamp=None):
        """

        :param value_name:
        :param value:
        :param host_timestamp:
        """
        super(ValueMessage, self).__init__(value_name, host_timestamp)
        self._value = value

    @property
    def value_name(self):
        return self.content

    @property
    def value(self):
        return self._value

    def tolist(self):
        return [
            self.MESSAGE_TYPE_ALIAS,
            str(self.pc_timestamp),
            self.host_timestamp,
            None,
            self.value_name,
            self.value,
        ]

    @classmethod
    def fromlist(cls, row):
        """
        Returns True if the typestr represents the class
        """
        obj = cls(int(row[4]), row[5], float(row[2]) if row[2] else None)
        obj.pc_timestamp = date_parser.parse(row[1])

        return obj



# !/usr/bin/python3
# -*- coding: utf-8 -*-

""" pycontrol.api.models.project

"""
import logging
import uuid
import os
import shutil
from pybpodgui_api.models.experiment import Experiment
from pybpodgui_api.models.board import Board
from pybpodgui_api.models.task import Task
from pybpodgui_api.models.subject import Subject
from pybpodgui_api.models.user import User
from pybpodgui_api.utils.copy_directory import copy_directory

logger = logging.getLogger(__name__)


class ProjectBase(object):
    """
    A project is a collection of experiments and an hardware configuration
    """

    def __init__(self):
        self.uuid4 = uuid.uuid4()
        self.name = ''
        self._experiments = []
        self._tasks = []
        self._boards = []
        self._subjects = []
        self._users = []
        self._path = None

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def subjects(self):
        """
        Get the list of subjects in the project

        :rtype: list(Subject)
        """
        return self._subjects

    @property
    def experiments(self):
        """
        Get the list of experiments in the project

        :rtype: list(Experiment)
        """
        return self._experiments

    @property
    def boards(self):
        """
        Get the list of boards in the project

        :rtype: list(Board)
        """
        return self._boards

    @property
    def tasks(self):
        """
        Get the list of tasks in the project

        :rtype: list(Task)
        """
        return self._tasks

    @property
    def path(self):
        """
        Get and set the project path

        :rtype: str
        """
        return self._path

    @property
    def users(self):
        """
        Get the list of users in the project

        :rtype: list(User)
        """
        return self._users

    @path.setter
    def path(self, value):
        self._path = value

    @property
    def name(self):
        """
        Get and set the project name

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def import_task(self, filepath, importdir=False):
        if self.path is None:
            raise Exception('The project has to be saved first')

        filename, file_extension = os.path.splitext(os.path.basename(filepath))

        # check if there are any existing task with the same name
        if self.find_task(filename) is not None:
            raise Exception(
                """There is already a task named [{}].The import was aborted""".format(filename),
            )
        ###########################################################

        # create the task, folder, and copy the files
        task = self.create_task()
        task.name = filename
        task.make_path()
        new_filepath = os.path.join(task.path, task.name+'.py')
        task.filepath = new_filepath

        if importdir:
            copy_directory(os.path.dirname(filepath), task.path + "/")
        else:
            shutil.copy(filepath, new_filepath)
        ###########################################################

        return task

    def __add__(self, obj):
        if isinstance(obj, Experiment):
            self._experiments.append(obj)

        if isinstance(obj, Board):
            self._boards.append(obj)

        if isinstance(obj, Task):
            self._tasks.append(obj)

        if isinstance(obj, Subject):
            self._subjects.append(obj)

        if isinstance(obj, User):
            self._users.append(obj)

        return self

    def __sub__(self, obj):
        if isinstance(obj, Experiment):
            self._experiments.remove(obj)

        if isinstance(obj, Board):
            self._boards.remove(obj)

        if isinstance(obj, Task):
            self._tasks.remove(obj)

        if isinstance(obj, Subject):
            self._subjects.remove(obj)

        if isinstance(obj, User):
            self._users.remove(obj)

        return self

    def find_board(self, name):
        """
        Find a board by the name

        :ivar str name: Name of the board to find.
        :rtype: Board
        """
        for board in self.boards:
            if board.name == name:
                return board
        return None

    def find_task(self, name):
        """
        Find a task by the name

        :ivar str name: Name of the task to find.
        :rtype: Task
        """
        for task in self.tasks:
            if task.name == name:
                return task
        return None

    def find_subject(self, name):
        """
        Find a subject by the name

        :ivar str name: Name of the subject to find.
        :rtype: Subject
        """
        for subject in self.subjects:
            if subject.name == name:
                return subject
        return None

    def find_setup_by_id(self, uuid4):
        """
        Find a setup by the id

        :ivar str uuid4: UUID4 of the setup to find.
        :rtype: Setup
        """
        for experiment in self.experiments:
            for setup in experiment.setups:
                if setup.uuid4 == uuid4:
                    return setup
        return None

    def find_subject_by_id(self, uuid4):
        """
        Find a subject by the name

        :ivar str name: Name of the subject to find.
        :rtype: Subject
        """
        for subject in self.subjects:
            if subject.uuid4 == uuid4:
                return subject
        return None

    def find_session(self, uuid4):
        for experiment in self.experiments:
            for setup in experiment.setups:
                for session in setup.sessions:
                    if session.uuid4 == uuid4:
                        return session
        return None

    def find_user(self, username):
        for user in self.users:
            if user.name == username:
                return user
        return None

    def create_experiment(self):
        """
        Add an experiment to the project, and return it.

        :rtype: Experiment
        """
        return Experiment(self)

    def create_board(self):
        """
        Add an board to the project, and return it.

        :rtype: Board
        """
        return Board(self)

    def create_task(self):
        """
        Add an task to the project, and return it.

        :rtype: Task
        """
        return Task(self)

    def create_subject(self):
        """
        Add an subject to the project, and return it.

        :rtype: Subject
        """
        return Subject(self)

    def create_user(self):
        """
        Add a user bject to the project, and return it.

        :rtype: User
        """
        return User(self)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os
import uuid

from pybpodgui_api.utils.generate_name import generate_name

from .taskcommand import ExecCmd, ScriptCmd

logger = logging.getLogger(__name__)


class TaskBase(object):
    """ Represents a state machine """

    def __init__(self, project=None):
        """
        :ivar Project project: Project to which the Task belongs to.
        """
        self.trigger_softcodes = False

        self.uuid4 = uuid.uuid4()
        self.filepath = None
        self.project = project
        self.name = generate_name([x.name for x in project.tasks], "task") if project else None
        self.project += self

        self._commands = []

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def name(self):
        """
        Get and set task name

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        previous_name = self._name if hasattr(self, '_name') else None
        previous_path = self.path if hasattr(self, '_name') else None

        self._name = value
        if previous_path is not None and self.filepath is not None:
            filepath = os.path.join(previous_path, self.name+'.py')
            os.rename(self.filepath, filepath)

            oldfilepath = os.path.join(previous_path, previous_name+'.json')
            filepath = os.path.join(previous_path, self.name+'.json')
            if os.path.isfile(oldfilepath):
                os.rename(oldfilepath, filepath)

            os.rename(previous_path, self.path)
            self.filepath = os.path.join(self.path, self.name+'.py')

    @property
    def path(self):
        """
        Get and set task path

        :rtype: str
        """
        if self.project.path is None:
            return None
        return os.path.join(self.project.path, 'tasks', self.name)

    @property
    def filepath(self):
        """
        Get and set task file path

        :rtype: str
        """
        return self._filepath

    @filepath.setter
    def filepath(self, value):
        self._filepath = value

    @property
    def project(self):
        """
        Get and set project

        :rtype: Project
        """
        return self._project

    @project.setter
    def project(self, project):
        self._project = project

    @property
    def trigger_softcodes(self):
        """
        Get net port

        :rtype: int
        """
        return self._trigger_softcodes

    @trigger_softcodes.setter
    def trigger_softcodes(self, value):
        self._trigger_softcodes = value

    @property
    def commands(self):
        """
        Get commands

        :rtype: list(TaskCommand)
        """
        return self._commands

    @commands.setter
    def commands(self, commands):
        """
        Setter for commands
        :param commands:
        :return:
        """
        self._commands = commands

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def make_path(self):
        """
        Creates the task folder
        """
        tasks_path = os.path.join(self.project.path, 'tasks')
        if not os.path.exists(tasks_path):
            os.makedirs(tasks_path)
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        open(os.path.join(self.path, '__init__.py'), 'w').close()

    def make_emptyfile(self):
        """
        Creates the task folder if does not exists and an empty code file
        """
        self.make_path()
        filepath = os.path.join(self.path, self.name+'.py')
        open(filepath, 'w').close()
        return filepath

    def create_scriptcmd(self):
        return ScriptCmd(self)

    def create_execcmd(self):
        return ExecCmd(self)

    def __add__(self, obj):
        if isinstance(obj, ScriptCmd):
            self._commands.append(obj)

        if isinstance(obj, ExecCmd):
            self._commands.append(obj)

        return self

    def __sub__(self, obj):
        if isinstance(obj, ScriptCmd):
            self._commands.remove(obj)

        if isinstance(obj, ExecCmd):
            self._commands.remove(obj)

        return self

    def remove(self):
        """
        Remove the task from the project.
        """
        pass

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.__unicode__()



from confapp import conf
from pybpodgui_plugin.models.user.user_dockwindow import UserDockWindow

User = type(
    'User',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.user.User') + [UserDockWindow]),
    {}
)



# !/usr/bin/python3
# -*- coding: utf-8 -*-
from pybpodgui_api.models.session.session_io import SessionIO


class SessionCom(SessionIO):
    """

    **Properties**

        status
            :class:`int`

            Holds setup status

    **Methods**

    """

    #### SESSION STATUS CONSTANTS ####
    STATUS_READY = 0
    STATUS_SESSION_RUNNING = 1  # The session is running

    def __init__(self, setup):
        super(SessionCom, self).__init__(setup)
        self.status = self.STATUS_READY

    @property
    def status(self):
        """
        Get and set the session status

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

        if value == self.STATUS_READY:
            self.setup.status = self.setup.STATUS_READY
        elif value == self.STATUS_SESSION_RUNNING:
            self.setup.status = self.setup.STATUS_RUNNING_TASK



import requests
import json
from confapp import conf
from pybpod_alyx_module.alyxapi.subjects.subjects import Subjects
from pybpod_alyx_module.alyxapi.data.data import Data

class AlyxAPI():

    def __init__(self):
        self._logged = False
        self.subjects = Subjects(self)
        self.data = Data(self)
        self.addr = conf.ALYX_PLUGIN_ADDRESS
        #self.serveraddr = 'http://alyx.champalimaud.pt:8000'
        #self.auth_endpoint = '/auth-token/'

    def login(self, _username, _password):
        _data = dict(username = _username,password = _password)
        result = requests.post(self.addr+'/auth-token',data=_data)
        if result.ok:
            token = result.json()
            
            self.headers = {
                'Authorization': 'Token {}'.format(list(token.values())[0]),
                'Accept': 'application/json',
                'Content-Type': 'application/json',
            }
            return True
        return False

    def getusers(self):
        result = requests.get(self._addr+'/users',self.headers)


    def getaddr(self):
        return self.addr

    def setaddr(self,value):
        self.addr = value





# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from pybpodgui_api.models.setup.task_variable import TaskVariable

logger = logging.getLogger(__name__)


class BoardTask(object):
    """
    Represents the association between one board and one task
    """

    def __init__(self, setup):
        """
        :ivar Setup setup: Setup to which the BoardTask belongs to
        """
        self.setup = setup
        self.board = None
        self.task = None
        self.variables = []
        self.update_variables = False

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def board(self):
        """
        Get and set the board

        :rtype: Board
        """
        return self._board

    @board.setter
    def board(self, value):
        self._board = value

    @property
    def task(self):
        """
        Get and set the Task

        :rtype: Task
        """
        return self._task

    @task.setter
    def task(self, value):
        self._task = value

    @property
    def variables(self):
        """
        Get and set list of variables

        :rtype: list(TaskVariable)
        """
        return self._variables

    @variables.setter
    def variables(self, value):
        self._variables = value

    @property
    def update_variables(self):
        """
        Get and set a flag that indicates if the variables should be updated
        at the end of the session

        :rtype: bool
        """
        return self._update_variables

    @update_variables.setter
    def update_variables(self, value):
        self._update_variables = value

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def create_variable(self, name=None, value=None, datatype='string'):
        """
        Create a variable

        :rtype: TaskVariable
        """
        return TaskVariable(self, name, value, datatype)

    def collect_data(self, data):
        data.update({'variables': [var.collect_data({}) for var in self.variables]})
        return data

    def save(self):
        """
        Save board task data on filesystem.

        :ivar str setup_path: Setup path.
        :return: Dictionary containing the board task info to save.
        :rtype: dict
        """
        return {
            'variables': [var.save() for var in self.variables],
            'update-variables': self.update_variables
        }

    def load(self, data):
        """
        Load setup data from filesystem

        :ivar str setup_path: Path of the setup
        :ivar dict data: data object that contains all setup info
        """
        self.update_variables = data.get('update-variables', False)
        for var_data in data.get('variables', []):
            var = self.create_variable()
            var.load(var_data)

    def __unicode__(self):
        return "Board : {board} | Task: {task}".format(board=str(self.board), task=str(self.task))

    def __str__(self):
        return self.__unicode__()

    def __add__(self, other):
        if isinstance(other, TaskVariable):
            self.variables.append(other)
        return self



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class BaseEventName(object):
    @staticmethod
    def is_state_timer(event_name):
        """

        :param str event_name:
        :rtype: bool
        """
        return True if event_name.startswith("Tup") else False

    @staticmethod
    def is_condition(event_name):
        """

        :param str event_name:
        :rtype: bool
        """
        return True if event_name.startswith("Condition") else False

    @staticmethod
    def is_global_counter_end(event_name):
        """

        :param str event_name:
        :rtype: bool
        """
        return (
            True
            if event_name.startswith("GlobalCounter") and event_name.endswith("End")
            else False
        )

    @staticmethod
    def is_global_timer_trigger(event_name):
        """
        :param str event_name:
        :rtype: bool
        """
        return event_name == "GlobalTimerTrig"

    @staticmethod
    def is_global_timer_cancel(event_name):
        """

        :param str event_name:
        :rtype: bool
        """
        return event_name == "GlobalTimerCancel"

    @staticmethod
    def is_global_timer_start(event_name):
        """

        :param str event_name:
        :rtype: bool
        """
        return (
            True
            if event_name.startswith("GlobalTimer") and event_name.endswith("Start")
            else False
        )

    @staticmethod
    def is_global_timer_end(event_name):
        """

        :param str event_name:
        :rtype: bool
        """
        return (
            True
            if event_name.startswith("GlobalTimer") and event_name.endswith("End")
            else False
        )



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from pybpodapi.com.messaging.base_message import BaseMessage


class DebugMessage(BaseMessage):

    """ Information line for things like experiment name , task name, board id, etc. """

    MESSAGE_TYPE_ALIAS = "debug"
    MESSAGE_COLOR = (200, 200, 200)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pybpodgui_plugin.models.setup.setup_treenode import SetupTreeNode

logger = logging.getLogger(__name__)


class SetupDockWindow(SetupTreeNode):
    """
    Dock window settings.
    Define here behaviors associated with board dock window.

    **Properties**

        mainwindow
            Returns project main window.

    **Methods**

    """
    MARKED_FOR_REMOVAL = False

    def __init__(self, experiment):
        super(SetupDockWindow, self).__init__(experiment)

    def show(self):
        """
        Select this window as the main window on the details section.
        Also reload boards list on combo box.
        """
        self.mainwindow.details.value = self
        self.reload_boards(current_selected_board=self.board)
        self.reload_tasks(current_selected_task=self.task)
        super(SetupDockWindow, self).show()

    def focus_name(self):
        """
        Sets interface focus on the board name text field
        """
        self._name.form.lineEdit.setFocus()

    def remove(self):
        """

        Prompts user to confirm setup removal.

        .. seealso::
            This method extends setup tree node :py:meth:`pybpodgui_plugin.models.setup.setup_treenode.SetupTreeNode.remove`.

        """
        if self.experiment.MARKED_FOR_REMOVAL:
            reply = 'yes'
        else:
            reply = self.question('Setup "{0}" and all the sessions will be deleted. Are you sure?'.format(self.name),
                                  "Warning")
        if reply == 'yes':
            self.MARKED_FOR_REMOVAL = True
            self.mainwindow.details.value = None
            super(SetupDockWindow, self).remove()

    @property
    def mainwindow(self):
        return self.experiment.mainwindow



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf

from pybpodgui_plugin.models.session.session_uibusy import SessionUIBusy

Session = type(
    'Session',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.session.Session') + [SessionUIBusy]), {}
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

""" pycontrolgui_plugin_timeline.trials_plot"""

from pyforms.controls import ControlEventsGraph
from pyforms.controls import ControlCheckBoxList
from pyforms.controls import ControlButton
from pyforms.basewidget import BaseWidget

from pybpodapi.com.messaging.state_occurrence import StateOccurrence
from pybpodapi.com.messaging.event_occurrence import EventOccurrence
from pybpodapi.com.messaging.event_resume import EventResume
from pybpodapi.com.messaging.session_info import SessionInfo
from pybpodgui_api.exceptions.run_setup import RunSetupError

import logging
import math
from datetime import datetime as datetime_now

from confapp import conf

from AnyQt.QtCore import QTimer, QEventLoop

logger = logging.getLogger(__name__)


class TrialsPlotWindow(BaseWidget):
    """ Show all boxes live state for an experiment"""

    EVENTS_TO_SHOW = [
        StateOccurrence,
        EventOccurrence
    ]

    def __init__(self, session):
        """
        :param session: session reference
        :type session: pycontrolgui.windows.detail.entities.session_window.SessionWindow
        """
        BaseWidget.__init__(self, session.name)
        self.session = session

        self._refreshbtn = ControlButton('Refresh Timeline')
        self._list = ControlCheckBoxList('Events to show')
        self._events = ControlEventsGraph(session.name)

        self._list.hide()
        self._events.add_popup_menu_option(
            'Show / Hide Events', self.__toggle_events_to_show_evt)
        self._events.add_track('')
        # for state_id, state_name in sorted(self.session.setup.board_task.states.items(), key=lambda x: x[0]):
        #	self._events.add_track(state_name)

        self._history_index = 0
        self._last_event = None
        self._session_start_timestamp = datetime_now.now()

        self.formset = ['_refreshbtn', '_list', '=', '_events']

        self._list_of_states_colors = [
            '#E0E0E0', '#FFCC99', '#FFFF99', 'CCFF99', '#99FFFF', '#99CCFF', '#FF99CC']

        self._states_names = {}

        self._timer = QTimer()
        self._timer.timeout.connect(self.read_message_queue)

        self._refreshbtn.value = self.__refresh_evt
        self._list.value = [
            (evt_type.__name__, True) for evt_type in self.EVENTS_TO_SHOW]
        self._list.changed_event = self.__list_changed_evt

        self._available_events = dict([
            (evt_type.__name__, evt_type) for evt_type in self.EVENTS_TO_SHOW])

        self.__list_changed_evt()

    def __refresh_evt(self):
        """Clears the entire timeline and re-draws all the checked events from T = 0 """
        self._events.clear()
        self._history_index = 0
        self.read_message_queue()

    def __list_changed_evt(self):
        self._events_2_draw = tuple([self._available_events[event_name]
                                     for event_name in self._list.value] + [SessionInfo])

    def __toggle_events_to_show_evt(self):
        if self._list.visible:
            self._list.hide()
        else:
            self._list.show()

    def show(self):
        # Prevent the call to be recursive because of the mdi_area
        if hasattr(self, '_show_called'):
            BaseWidget.show(self)
            return
        self._show_called = True
        self.mainwindow.mdi_area += self
        del self._show_called

        self._stop = False  # flag used to close the gui in the middle of a loading
        self.read_message_queue()
        if not self._stop:
            self._timer.start(conf.TIMELINE_PLUGIN_REFRESH_RATE)

    def hide(self):
        self._timer.stop()
        self._stop = True

    def before_close_event(self):
        self._timer.stop()
        self._stop = True
        self.session.trialsplot_action.setEnabled(True)

    def __add_event(self, start_timestamp, end_timestamp, track_id, name):

        self._last_event = self._events.add_event(
            start_timestamp,
            end_timestamp,
            track=track_id,
            title=name,
            color=self._list_of_states_colors[track_id % len(
                self._list_of_states_colors)]
        )
        self._events.value = start_timestamp

    def timediff_ms(self, time_f, time_i):
        diff = datetime_now.now()

        diff = time_f - time_i
        elapsed_ms = (diff.days * 86400000) + \
            (diff.seconds * 1000) + (diff.microseconds / 1000)
        return elapsed_ms

    def read_message_queue(self):
        """ Update board queue and retrieve most recent messages """

        try:
            recent_history = self.session.messages_history[self._history_index:]

            for message in recent_history:

                if self._stop:
                    return

                if not isinstance(message, self._events_2_draw):
                    continue

                if isinstance(message, StateOccurrence):
                    if message.state_name not in self._states_names.keys():
                        self._states_names[message.state_name] = len(
                            self._states_names)

                    if not (math.isnan(message.start_timestamp) or math.isnan(message.end_timestamp)):
                        bpod_start = int(round(message.start_timestamp * 1000))
                        bpod_end = int(round(message.end_timestamp * 1000))
                        self.__add_event(
                            bpod_start,
                            bpod_end,
                            self._states_names[message.state_name],
                            message.state_name
                        )

                elif isinstance(message, SessionInfo):
                    if message.content == 'SESSION-STARTED':
                        self._session_start_timestamp = message.pc_timestamp

                elif isinstance(message, EventOccurrence):
                    ts = None  # in case we don't find any valid timestamp, don't add the event to the timeline
                    # check which timestamp will be used. host timestamp wins over pc timestamp

                    if message.pc_timestamp is not None:
                        ts = self.timediff_ms(
                            message.pc_timestamp, self._session_start_timestamp)

                    # proceed if we have a valid timestamp
                    if ts is not None:
                        # create a new event slot in the timeline in case current message is entirely new
                        if message.event_name not in self._states_names.keys():
                            self._states_names[message.event_name] = len(
                                self._states_names)
                        # add a time delta to ts so the event can be shown in the timeline
                        self.__add_event(
                            ts - 10, ts + 10, self._states_names[message.event_name], message.event_name)

                self._history_index += 1
                QEventLoop()

        except RunSetupError as err:
            logger.error(str(err), exc_info=True)
            self._timer.stop()

    @property
    def mainwindow(self):
        return self.session.mainwindow

    @property
    def title(self):
        return BaseWidget.title.fget(self)

    @title.setter
    def title(self, value):
        title = 'Trials-plot: {0}'.format(value)
        BaseWidget.title.fset(self, title)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import serial
import numpy as np
import struct

logger = logging.getLogger(__name__)


class DataType(object):
    def __init__(self, name, size):
        self.name = name
        self.size = size

    def __str__(self):
        return self.name


class ArduinoTypes(object):
    BYTE = DataType("byte", 1)
    CHAR = DataType("char", 1)
    UINT8 = DataType("uint8", 1)
    INT16 = DataType("int16", 2)
    UINT16 = DataType("uint16", 2)
    UINT32 = DataType("uint32", 4)
    UINT64 = DataType("uint64", 8)
    FLOAT32 = DataType("float32", 4)
    FLOAT64 = DataType("float64", 8)

    @staticmethod
    def get_array(array, dtype):
        if dtype == ArduinoTypes.CHAR or dtype == ArduinoTypes.UINT8:
            return ArduinoTypes.get_uint8_array(array)
        elif dtype == ArduinoTypes.UINT16:
            return ArduinoTypes.get_uint16_array(array)
        elif dtype == ArduinoTypes.UINT32 or dtype == ArduinoTypes.FLOAT:
            return ArduinoTypes.get_uint32_array(array)
        else:
            return None

    @staticmethod
    def get_uint8_array(array):
        return np.array(array, dtype=str(ArduinoTypes.UINT8)).tobytes()

    @staticmethod
    def get_int16_array(array):
        return np.array(array, dtype=str(ArduinoTypes.INT16)).tobytes()

    @staticmethod
    def get_uint16_array(array):
        return np.array(array, dtype=str(ArduinoTypes.UINT16)).tobytes()

    @staticmethod
    def get_uint32_array(array):
        return np.array(array, dtype=str(ArduinoTypes.UINT32)).tobytes()

    @staticmethod
    def get_float(value):
        return struct.pack("<f", value)

    @staticmethod
    def cvt_float32(message_bytes):
        return struct.unpack("<f", message_bytes)[0]

    @staticmethod
    def cvt_float64(message_bytes):
        return struct.unpack("<d", message_bytes)[0]

    @staticmethod
    def cvt_int64(message_bytes):
        return int.from_bytes(message_bytes, byteorder="little")

    @staticmethod
    def cvt_uint32(message_bytes):
        return struct.unpack("<L", message_bytes)[0]

    @staticmethod
    def cvt_uint64(message_bytes):
        return struct.unpack("<Q", message_bytes)[0]


class ArCOM(object):
    """
    ArCOM is an interface to simplify data transactions between Arduino and Python.
    """

    def open(self, serial_port, baudrate=115200, timeout=1):
        """
        Open serial connection
        :param serialPortName:
        :param baudRate:
        :return:
        """
        self.serial_object = serial.Serial(
            serial_port, baudrate=baudrate, timeout=timeout
        )

        return self

    def close(self):
        """
        Close serial connection
        :return:
        """
        self.serial_object.close()

    def bytes_available(self):
        """

        :return:
        """
        return self.serial_object.inWaiting()

    ##############################################################
    ## WRITE #####################################################
    ##############################################################

    def write_char(self, value):
        self.serial_object.write(str.encode(value))

    def write_array(self, array):
        self.serial_object.write(array)

    ##############################################################
    ## READ BYTE #################################################
    ##############################################################

    def read_byte(self):
        message_bytes = self.serial_object.read(ArduinoTypes.BYTE.size)
        return message_bytes

    def read_char(self):
        message_bytes = self.serial_object.read(ArduinoTypes.CHAR.size)

        return message_bytes.decode("utf-8")

    def read_uint8(self):
        message_bytes = self.serial_object.read(ArduinoTypes.UINT8.size)
        # logger.debug("Read %s bytes: %s", len(message_bytes), message_bytes)
        message = int.from_bytes(message_bytes, byteorder="little")
        return message

    def read_uint16(self):
        message_bytes = self.serial_object.read(ArduinoTypes.UINT16.size)
        # logger.debug("Read %s bytes: %s", ArduinoTypes.UINT16.size, message_bytes)
        message = int.from_bytes(message_bytes, byteorder="little")
        return message

    def read_uint32(self):
        message_bytes = self.serial_object.read(ArduinoTypes.UINT32.size)
        # logger.debug("Read %s bytes: %s", ArduinoTypes.UINT32.size, message_bytes)
        message = int.from_bytes(message_bytes, byteorder="little")
        return message

    def read_uint64(self):
        message_bytes = self.serial_object.read(ArduinoTypes.UINT64.size)
        # logger.debug("Read %s bytes: %s", ArduinoTypes.UINT32.size, message_bytes)
        message = int.from_bytes(message_bytes, byteorder="little")
        return message

    def read_float32(self):
        message_bytes = self.serial_object.read(ArduinoTypes.FLOAT32.size)
        # logger.debug("Read %s bytes: %s", ArduinoTypes.UINT32.size, message_bytes)
        message = struct.unpack("<f", message_bytes)
        return message[0]

    ##############################################################
    ## READ ARRAY ################################################
    ##############################################################

    def read_bytes_array(self, array_len=1):
        message_array = []
        for pos in range(0, array_len):
            message_bytes = self.read_byte()
            message_array.append(message_bytes)
        return message_array

    def read_char_array(self, array_len=1):
        message_array = []
        for pos in range(0, array_len):
            message_bytes = self.read_char()
            message_array.append(message_bytes)

        return message_array

    def read_uint8_array(self, array_len=1):
        message_array = []
        for pos in range(0, array_len):
            message_bytes = self.read_uint8()
            message_array.append(message_bytes)

        return message_array

    def read_uint16_array(self, array_len=1):
        message_array = []
        for pos in range(0, array_len):
            message_bytes = self.read_uint16()
            message_array.append(message_bytes)

        return message_array

    def read_uint32_array(self, array_len=1):
        message_array = []
        for pos in range(0, array_len):
            message_bytes = self.read_uint32()
            message_array.append(message_bytes)

        return message_array

    def read_float32_array(self, array_len=1):
        message_array = []
        for pos in range(0, array_len):
            message_bytes = self.read_float32()
            message_array.append(message_bytes)

        return message_array



# !/usr/bin/python3
# -*- coding: utf-8 -*-


import logging, os, sys

if 'terminal_mode' in sys.argv:
	PYFORMS_MODE = 'TERMINAL'
else:
	PYFORMS_MODE = os.environ.get('PYFORMS_MODE', 'GUI')

PYFORMS_LOG_FILENAME 				= 'pyforms.log'
PYFORMS_LOG_HANDLER_FILE_LEVEL 		= logging.INFO
PYFORMS_LOG_HANDLER_CONSOLE_LEVEL 	= logging.INFO


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from AnyQt.QtGui import QIcon

from pybpodgui_api.models.setup import Setup
from pybpodgui_plugin.models.experiment.experiment_dockwindow import ExperimentDockWindow

logger = logging.getLogger(__name__)


class ExperimentUIBusy(ExperimentDockWindow):
    """
    Extends experiment with UI refreshment logic.

    .. seealso::
        UI buttons: :py:class:`pybpodgui_plugin.models.experiment.experiment_window.ExperimentWindow`.

    **Methods**

    """

    def __init__(self, project):
        super(ExperimentUIBusy, self).__init__(project)
        self.__running_icon = QIcon(conf.PLAY_SMALL_ICON)

    ############################################################################

    def update_ui(self):
        """
        Update UI button states and labels and tree icons when board communication is active.
        """
        busy_status = Setup.STATUS_READY

        # Check first if any setup is busy
        for setup in self.setups:
            if setup.status > Setup.STATUS_BOARD_LOCKED:
                busy_status = setup.status
                break

        # if there is no setups busy, will test if the boards are locked
        if busy_status == Setup.STATUS_READY:
            for setup in self.setups:
                if setup.status > Setup.STATUS_READY:
                    busy_status = setup.status
                    break

        logger.debug('Experiment [{0}] status:{1}'.format(self.name, busy_status))

        if busy_status == Setup.STATUS_READY:

            self.node.setIcon(0, QIcon(conf.EXPERIMENT_SMALL_ICON))
            # if self.task:
            # 	self.enable_all_task_buttons()
            # else:
            # 	self.disable_all_task_buttons()
            # self._task.enabled = True

        elif busy_status == Setup.STATUS_BOARD_LOCKED:

            self.node.setIcon(0, QIcon(conf.BUSY_SMALL_ICON))
            self.disable_all_task_buttons()
            # self._task.enabled = False

        elif busy_status == Setup.STATUS_RUNNING_TASK:

            self.node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            self.disable_all_task_buttons()
            # self._task.enabled = False

        for setup in self.setups:
            setup.update_ui()

    def enable_all_task_buttons(self):
        """
        Enable UI buttons for restore task variables, upload task and run task
        """
        # self._button_run_all.enabled = True
        pass

    def disable_all_task_buttons(self):
        """
        Disable UI buttons for restore task variables, upload task and run task
        """
        # self._button_run_all.enabled = False
        pass



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class OutputChannel(object):
    """
    Available output channels
    These values must be set according to Bpod firmware specification.
    """

    #: LED
    LED = "LED"

    #: Valve
    Valve = "Valve"

    #: Serial 1
    Serial1 = "Serial1"

    #: Serial 2
    Serial2 = "Serial2"

    #: Serial 3
    Serial3 = "Serial3"

    #: SoftCode
    SoftCode = "SoftCode"

    #: ValveState
    ValveState = "ValveState"

    #: BNC1
    BNC1 = "BNC1"

    #: BNC2
    BNC2 = "BNC2"

    #: Wire1
    Wire1 = "Wire1"

    #: Wire2
    Wire2 = "Wire2"

    #: Wire3
    Wire3 = "Wire3"

    #: Wire3
    Wire4 = "Wire4"

    #: PWM1
    PWM1 = "PWM1"

    #: PWM2
    PWM2 = "PWM2"

    #: PWM3
    PWM3 = "PWM3"

    #: PWM4
    PWM4 = "PWM4"

    #: PWM5
    PWM5 = "PWM5"

    #: PWM6
    PWM6 = "PWM6"

    #: PWM7
    PWM7 = "PWM7"

    #: PWM8
    PWM8 = "PWM8"

    #: GlobalTimerTrig
    GlobalTimerTrig = "GlobalTimerTrig"

    #: GlobalTimerCancel
    GlobalTimerCancel = "GlobalTimerCancel"

    #: GlobalCounterReset
    GlobalCounterReset = "GlobalCounterReset"



# !/usr/bin/python3
# -*- coding: utf-8 -*-
import logging
import inspect
from confapp import conf

from AnyQt.QtGui import QIcon
from AnyQt import QtCore

from pybpodgui_plugin.models.subject.subject_window import SubjectWindow
from pybpodgui_plugin.models.session import Session

logger = logging.getLogger(__name__)


class SubjectTreeNode(SubjectWindow):
    """
    Extends board window to show up in the project tree section.
    Define here actions related to the board tree node.

    **Properties**

        name
            Handles board tree node name.

        tree
            Returns project tree.

    **Methods**

    """

    def __init__(self, project):
        SubjectWindow.__init__(self, project)
        self.__running_icon = QIcon(conf.PLAY_SMALL_ICON)
        self.create_treenode(self.tree)

    def create_treenode(self, tree):
        """
        Creates node for this board under the parent "Boards" node.

        This methods is called when the board is first created.

        The following actions get assigned to node:
            * *Remove*: :meth:`BoardTreeNode.remove`.

        Sets key events:
            * :meth:`BoardTreeNode.node_key_pressed_event`


        :param tree: the project tree
        :type tree: pyforms.controls.ControlTree
        :return: new created node
        :return type: QTreeWidgetItem
        """
        self.node = tree.create_child(self.name, self.project.subjects_node, icon=QIcon(conf.SUBJECT_SMALL_ICON))
        self.node.key_pressed_event = self.node_key_pressed_event
        self.node.window = self
        self.node.setExpanded(True)

        tree.add_popup_menu_option('Remove', self.remove, item=self.node, icon=QIcon(conf.REMOVE_SMALL_ICON))
        return self.node

    def remove(self, silent=False):
        """

        Remove subject from project and remove node from tree.

        .. seealso::
            * Subject removal (dock window): :py:meth:`pybpodgui_plugin.models.subject.subject_dockwindow.SubjectDockWindow.remove`.
            * Subject removal (API): :meth:`pybpodgui_api.models.subject.subject_base.SubjectBase.remove`.
            * Remove board from project: :meth:`pybpodgui_api.models.project.project_base.ProjectBase.__sub__`.

        """
        self.project -= self
        self.project.subjects_node.removeChild(self.node)

    def node_key_pressed_event(self, event):
        """
        Sets key events for:
            * Remove board: :meth:`BoardTreeNode.remove`

        :param event: key event
        """
        if event.key() == QtCore.Qt.Key_Delete:
            self.remove()

    def create_sessiontreenode(self, session):
        node = self.tree.create_child(session.name, self.node)
        node.key_pressed_event = session.node_key_pressed_event
        node.double_clicked_event = session.node_double_clicked_event
        # This makes the sesison window appear correctly
        node.window = session.node.window
        session.subjects_nodes[id(self.node)] = node

        return node

    def __add__(self, session):
        if isinstance(session, Session):
            # add another node to the UI
            node = self.create_sessiontreenode(session)
            session.subjects_nodes[id(self.node)] = node
            self.tree.add_popup_menu_option('Remove', session.remove, item=node, icon=QIcon(conf.REMOVE_SMALL_ICON))
        return super(SubjectTreeNode, self).__add__(session)

    def __sub__(self, value):
        if isinstance(value, Session):
            self.node.removeChild(value.subjects_nodes[id(self.node)])
        return super(SubjectTreeNode, self).__sub__(value)

    @property
    def name(self):
        if hasattr(self, 'node'):
            return str(self.node.text(0))
        else:
            return SubjectWindow.name.fget(self)

    @name.setter
    def name(self, value):
        SubjectWindow.name.fset(self, value)
        if hasattr(self, 'node'):
            self.node.setText(0, value)

    @property
    def tree(self):
        return self.project.tree



# !/usr/bin/python3from pybpodapi.com.messaging.base_message import BaseMessage
# -*- coding: utf-8 -*-
from pybpodapi.com.messaging.base_message import BaseMessage


class SoftcodeOccurrence(BaseMessage):
    """
    Message from board that represents state change (an event)

    :ivar str event_name: name of the event
    :ivar int event_id: index of the event
    :ivar float board_timestamp: timestamp associated with this event (from bpod)

    """

    MESSAGE_TYPE_ALIAS = "SOFTCODE"
    MESSAGE_COLOR = (40, 30, 30)

    def __init__(self, softcode, host_timestamp=None):
        """

        :param event_id:
        :param event_name:
        :param host_timestamp:
        """
        super(SoftcodeOccurrence, self).__init__(softcode, host_timestamp)

    @property
    def softcode(self):
        return self.content



from pybpodapi.exceptions.bpod_error import BpodErrorException
from pybpodapi.com.arcom import ArduinoTypes


class BpodModule(object):
    def __init__(
        self,
        connected=False,
        module_name="",
        firmware_version=0,
        events_names=[],
        n_serial_events=0,
        serial_port=None,
    ):
        self.name = module_name
        self.serial_port = serial_port
        self.connected = connected
        self.firmware_version = firmware_version
        self.event_names = events_names
        self.n_serial_events = n_serial_events

        self.relay_active = False

        self.bpod_modules = None

    def __str__(self):
        return "{0} (connected: {1})(firmware: {2})".format(
            self.name, self.connected, self.firmware_version
        )

    def load_message(self, msg, msg_id=None):
        """
        Load a message through bpod to the module and associate an ID to it.

        :param list(int) msg: Message to send
        :param int msg_id: Id of the message to use
        """
        if msg_id is None:
            for i in range(255):
                if not self.bpod_modules.bpod.msg_id_list[i]:
                    msg_id = i + 1
                    break

        self.bpod_modules.bpod.load_serial_message(self.serial_port, msg_id, msg)
        return msg_id

    def start_module_relay(self):
        if not self.bpod_modules.relay_is_active:
            self.bpod_modules.activate_module_relay(self)
            self.relay_active = True
        else:
            raise BpodErrorException(
                "Error: You must disable the active module relay before starting another one."
            )

    def stop_module_relay(self):
        self.bpod_modules.deactivate_module_relay(self)
        self.relay_active = False

    def __read(self, size=None, dtype=None):
        if not self.relay_active:
            raise BpodErrorException(
                "Error: you must start the module relay with start_moule_relay() before you can read bytes from a module"
            )

        if size is None:
            size = self.bpod_modules.bpod.data_available()
        return self.bpod_modules.module_read(self, size, dtype)

    def write_char_array(self, message):
        self.bpod_modules.module_write(self, message, ArduinoTypes.CHAR)

    def write_uint8_array(self, message):
        self.bpod_modules.module_write(self, message, ArduinoTypes.UINT8)

    def write_uint16_array(self, message):
        self.bpod_modules.module_write(self, message, ArduinoTypes.UINT16)

    def write_uint32_array(self, message):
        self.bpod_modules.module_write(self, message, ArduinoTypes.UINT32)

    def write_float_array(self, message):
        self.bpod_modules.module_write(self, message, ArduinoTypes.FLOAT)

    def read_char_array(self, size=None):
        self.__read(size, ArduinoTypes.CHAR)

    def read_uint8_array(self, size=None):
        self.__read(size, ArduinoTypes.UINT8)

    def read_uint16_array(self, size=None):
        self.__read(size, ArduinoTypes.UINT16)

    def read_uint32_array(self, size=None):
        self.__read(size, ArduinoTypes.UINT32)

    def read_float_array(self, size=None):
        self.__read(size, ArduinoTypes.FLOAT)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import pprint

from pybpodapi.com.messaging.event_occurrence import EventOccurrence
from pybpodapi.com.messaging.state_occurrence import StateOccurrence
from pybpodapi.com.messaging.base_message import BaseMessage
from pybpodapi.utils import date_parser

logger = logging.getLogger(__name__)


class Trial(BaseMessage):
    """
    :ivar float trial_start_timestamp: None
    :ivar StateMachine sma: sma
    :ivar list(StateOccurrence) states_occurrences: list of state occurrences
    :ivar list(EventOccurrence) events_occurrences: list of event occurrences
    """

    MESSAGE_TYPE_ALIAS = "TRIAL"
    MESSAGE_COLOR = (0, 0, 255)

    def __init__(self, sma=None):
        super(Trial, self).__init__("New trial")
        self.trial_start_timestamp = None
        self.sma = sma  # type: StateMachine
        self.states_occurrences = []  # type: list(StateOccurrence)
        self.events_occurrences = []  # type: list(EventOccurrence)

        self.states = [0]
        self.state_timestamps = [0]
        self.event_timestamps = []  # see also BpodBase.__update_timestamps

        self.states_durations = {}

    def __add__(self, msg):
        if isinstance(msg, EventOccurrence):
            self.events_occurrences.append(msg)
        elif isinstance(msg, StateOccurrence):
            self.states_occurrences.append(msg)
            if msg.state_name not in self.states_durations:
                self.states_durations[msg.state_name] = []
            self.states_durations[msg.state_name].append(
                (msg.start_timestamp, msg.end_timestamp)
            )

        return self

    def get_timestamps_by_event_name(self, event_name):
        """
        Get timestamps by event name

        :param event_name: name of the event to get timestamps
        :rtype: list(float)
        """
        event_timestamps = []  # type: list(float)

        for event in self.events_occurrences:
            name = self.sma.hardware.channels.get_event_name(event.event_id)
            if name == event_name:
                event_timestamps.append(event.host_timestamp)

        return event_timestamps

    def get_events_names(self):
        """
        Get events names without repetitions

        :rtype: list(str)
        """
        events_names = []  # type: list(str)

        for event in self.events_occurrences:
            event_name = self.sma.hardware.channels.get_event_name(event.event_id)
            if event_name not in events_names:
                events_names.append(event_name)

        return events_names

    def get_all_timestamps_by_event(self):
        """
        Create a dictionary whose keys are events names and values are corresponding timestamps

        Example:

        .. code-block:: python

            {
                'Tup': [429496.7295, 429496.7295],
                'Port3In': [429496.7295, 429496.7295],
                'Port2In': [429496.7295, 429496.7295],
                'Port2Out': [429496.7295, 429496.7295],
                'Port3Out': [429496.7295],
                'Port1Out': [429496.7295]
            }

        :rtype: dict
        """
        all_timestamps = {}
        for event_name in self.get_events_names():
            all_timestamps[event_name] = self.get_timestamps_by_event_name(event_name)

        return all_timestamps

    def export(self):
        return {
            "Bpod start timestamp": self.bpod_start_timestamp,
            "Trial start timestamp": self.trial_start_timestamp,
            "Trial end timestamp": self.trial_end_timestamp,
            "States timestamps": self.states_durations,
            "Events timestamps": self.get_all_timestamps_by_event(),
        }

    def pformat(self):
        return pprint.pformat(self.export(), indent=4)

    def __str__(self):
        return str(self.export())

    @classmethod
    def fromlist(cls, row):
        """
        Returns True if the typestr represents the class
        """
        obj = cls()
        obj.pc_timestamp = date_parser.parse(row[1])
        return obj



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os
import uuid
from pybpodgui_api.models.session import Session
from pybpodgui_api.utils.generate_name import generate_name

logger = logging.getLogger(__name__)


class SubjectBase(object):

    def __init__(self, project):
        self._path = None
        self.uuid4 = uuid.uuid4()

        self.name = generate_name([x.name for x in project.subjects], "subject")

        self.project = project
        self.setup = None

        self.project += self

        self._sessions = []

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    def __add__(self, value):
        if isinstance(value, Session):
            self._sessions.append(value)
        return self

    def __sub__(self, obj):
        if isinstance(obj, Session) and obj in self._sessions:
            self._sessions.remove(obj)

    def remove(self):
        print('subject base removing session')

    def get_sessions(self):
        """
        Get all subject sessions

        :rtype: list(Session)
        """
        print('subject base get sessions')
        for exp in self.project.experiments:
            for setup in exp.setups:
                for session in setup.sessions:
                    if self in session.subjects:
                        yield session

        return None

    @property
    def name(self):
        """
        Get and set setup name

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def project(self):
        """
        Get and set project

        :rtype: str
        """
        return self._project

    @project.setter
    def project(self, value):
        self._project = value

    @property
    def setup(self):
        """
        Get and set project

        :rtype: str
        """
        return self._setup

    @setup.setter
    def setup(self, value):
        self._setup = value

    @property
    def path(self):
        """
        Get and set the path

        :rtype: str
        """
        if self.project.path is None:
            return None
        return os.path.join(self.project.path, 'subjects', self.name)

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def remove(self):
        """
        Remove the subject from the project
        """
        pass

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.__unicode__()



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf
from pybpodgui_plugin.models.projects.projects_treenode import ProjectsTreeNode

Projects = type(
    'Projects',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.project.Projects') + [ProjectsTreeNode]),
    {}
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-


import logging
import os

import pyforms
from pyforms.basewidget import BaseWidget

logger = logging.getLogger(__name__)

try:
    from pyforms.controls import ControlCodeEditor
except:
    logger.error("Could not import ControlCodeEditor. Is QScintilla installed?")


class CodeEditor(BaseWidget):

    def __init__(self, board):
        BaseWidget.__init__(self, board.name)
        self.board = board
        self.layout().setContentsMargins(5, 5, 5, 5)

        self._code = ControlCodeEditor('Code', self.__get_code())

        self._code.changed_event = self.__code_changed_evt

    def __get_code(self):

        try:
            with open(self.board.hardware_file, "r") as file:
                return file.read()
        except:
            pass

        return ''

    def __code_changed_evt(self):
        hw_path = os.path.join(self.board.path, 'hardware.py')
        with open(hw_path, "w") as file:
            return file.write(self._code.value)

    def beforeClose(self):
        """
        Before closing window, ask user if she wants to save (if there are changes)
        """
        if self._code.is_modified:
            reply = self.question('Save the changes', 'Save the file')

            if reply == 'yes':
                self.__code_changed_evt()

        return False

    @property
    def title(self):
        return BaseWidget.title.fget(self)

    @title.setter
    def title(self, value):
        BaseWidget.title.fset(self, "{0} hardware editor".format(value))


# Execute the application
if __name__ == "__main__":
    pyforms.start_app(CodeEditor)



from datetime import datetime as datetime_now

from AnyQt import QtGui
from AnyQt.QtCore import QTimer
from AnyQt.QtWidgets import QFileDialog
from confapp import conf
from serial.tools.list_ports import comports

import pyforms
from pybpod_rotaryencoder_module.module_api import RotaryEncoderModule
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlMatplotlib
from pyforms.controls import ControlText, ControlCheckBox, ControlNumber, ControlButton, ControlCombo
from sca.formats import csv


class RotaryEncoderModuleGUI(RotaryEncoderModule, BaseWidget):

    TITLE = 'Rotary encoder module'

    def __init__(self, parent_win=None):
        BaseWidget.__init__(self, self.TITLE, parent_win=parent_win)
        RotaryEncoderModule.__init__(self)

        self._port = ControlCombo('Serial port', changed_event=self.__combo_serial_ports_changed_evt)
        self._refresh_serial_ports = ControlButton('',
                                                   icon=QtGui.QIcon(conf.REFRESH_SMALL_ICON),
                                                   default=self.__refresh_serial_ports_btn_pressed,
                                                   helptext="Press here to refresh the list of available devices.")

        self._connect_btn   = ControlButton('Connect', checkable=True)

        self._filename 		= ControlText('Stream Filename', '')
        self._saveas_btn    = ControlButton('Save As...')

        self._events 		= ControlCheckBox('Enable events')
        self._output_stream	= ControlCheckBox('Output stream')
        self._stream 		= ControlCheckBox('Stream data')
        self._stream_file   = ControlCheckBox('Stream to file')
        self._zero_btn 		= ControlButton('Reset position')
        self._start_reading = ControlButton('Start Reading')
        self._reset_threshs = ControlButton('Reset thresholds')
        self._thresh_lower 	= ControlNumber('Lower threshold (deg)', 0, minimum=-360, maximum=360)
        self._thresh_upper 	= ControlNumber('Upper threshold (deg)', 0, minimum=-360, maximum=360)
        self._graph 		= ControlMatplotlib('Value')
        self._clear_btn 	= ControlButton('Clear')

        self.set_margin(10)

        self.formset = [
            ('_port', '_refresh_serial_ports', '_connect_btn'),
            ('_filename', '_saveas_btn'),
            ('_events', '_output_stream', '_stream', '_stream_file', '_zero_btn'),
            '_start_reading',
            ('_thresh_lower', '_thresh_upper', '_reset_threshs'),
            '=',
            '_graph',
            '_clear_btn'
        ]

        self._stream.enabled = False
        self._stream_file.enabled = False
        self._events.enabled = False
        self._output_stream.enabled = False
        self._zero_btn.enabled = False
        self._reset_threshs.enabled = False
        self._thresh_lower.enabled = False
        self._thresh_upper.enabled = False
        self._start_reading.enabled = False

        self._connect_btn.value = self.__toggle_connection_evt
        self._saveas_btn.value = self.__prompt_savig_evt
        self._stream_file.changed_event = self.__stream_file_changed_evt
        self._events.changed_event = self.__events_changed_evt
        self._output_stream.changed_event = self.__output_stream_changed_evt
        self._thresh_upper.changed_event = self.__thresh_evt
        self._thresh_lower.changed_event = self.__thresh_evt
        self._reset_threshs.value = self.__reset_thresholds_evt
        self._zero_btn.value = self.__zero_btn_evt
        self._start_reading.value = self.__start_reading_evt
        self._graph.on_draw = self.__on_draw_evt
        self._clear_btn.value = self.__clear_btn_evt
        self._filename.changed_event = self.__filename_changed_evt

        self.history_x = []
        self.history_y = []

        self._timer = QTimer()
        self._timer.timeout.connect(self.__update_readings)

        self._fill_serial_ports()

    def _fill_serial_ports(self):
        self._port.add_item('', '')
        for n, port in enumerate(sorted(comports()), 1):
            self._port.add_item("{device}".format(device=port.device), str(port.device))

    def __filename_changed_evt(self):
        if not self._filename.value:
            self._stream_file.value = False
            self._stream_file.enabled = False

    def __prompt_savig_evt(self):
        '''
        Opens a window for user to select where to save the csv file
        '''
        self._filename.value, _ = QFileDialog.getSaveFileName()
        if self._filename.value:
            self._stream_file.enabled = True
        else:
            self._stream_file.value = False
            self._stream_file.enabled = False

    def __stream_file_changed_evt(self):
        '''
        User wants to store rotary encoder measurements in a CSV file. Create it
        '''
        if self._stream_file.value is True:
            self._csvfile = open(self._filename.value, 'w')
            self._csvwriter = csv.writer(
                self._csvfile,
                def_text='This file has all the rotary encoder data recorded during a PyBpod session.',
                columns_headers=['PC_TIME', 'DATA_TYPE','EVT_TIME', 'VALUE'])  # Check if we need something else after

    def __start_reading_evt(self):
        '''
        Toggle timer
        '''
        if self._timer.isActive():
            self.disable_stream()
            self._start_reading.label = 'Start Reading'
            self._timer.stop()
        else:
            self.enable_stream()
            self.history_x = []
            self.history_y = []
            self._start_reading.label = 'Stop Reading'
            self._timer.start(30)

    def __clear_btn_evt(self):
        '''
        Clear recorded data
        '''
        self.history_x = []
        self.history_y = []
        self._graph.draw()

    def __on_draw_evt(self, figure):
        '''
        The actual draw function. Pick just the last 200 measurements in order to avoid app freezing
        '''
        axes = figure.add_subplot(111)
        axes.clear()
        totallen = len(self.history_x)
        if totallen > 200:
            x = self.history_x[totallen - 201:]
            y = self.history_y[totallen - 201:]
            axes.plot(x,y)
            if len(x)>=2:
                x_range = [x[0],x[-1]]
                axes.plot(x_range, [self._thresh_upper.value, self._thresh_upper.value], linestyle='dotted', color='red')
                axes.plot(x_range, [self._thresh_lower.value, self._thresh_lower.value], linestyle='dotted', color='blue')
        else:
            axes.plot(self.history_x, self.history_y)
            if len(self.history_x) >= 2:
                x_range = [self.history_x[0], self.history_x[-1]]
                axes.plot(x_range, [self._thresh_upper.value, self._thresh_upper.value], linestyle='dotted', color='red')
                axes.plot(x_range, [self._thresh_lower.value, self._thresh_lower.value], linestyle='dotted', color='blue')

        self._graph.repaint()

    def __update_graph(self, readings):
        '''
        Add new data to the reading history and update the graph
        '''
        for data in readings:
            if data[0] == 'P':
                self.history_x.append(data[1])
                self.history_y.append(data[2])
        self._graph.draw()

    def __update_readings(self):
        '''
        Get new measurements and channel them to the graph or the file being written
        '''
        data = self.read_stream()

        if self._stream.value:
            self.__update_graph(data)
        if self._stream_file.value:
            self.__write_to_file(data)

    def __write_to_file(self, readings):
        '''
        Write new readings to the file
        '''
        now = datetime_now.now()
        for data in readings:
            self._csvwriter.writerow([now.strftime('%Y%m%d%H%M%S')]+data)

    def __zero_btn_evt(self):
        self.set_zero_position()

    def __reset_thresholds_evt(self):
        self._thresh_lower.value = 0
        self._thresh_upper.value = 0

    def __thresh_evt(self):
        thresholds = [int(self._thresh_lower.value), int(self._thresh_upper.value)]	
        self.set_thresholds(thresholds)

    def __events_changed_evt(self):
        if self._stream.value:
            self.enable_evt_transmission()
        else:
            self.disable_evt_transmission()

    def __output_stream_changed_evt(self):
        if self._stream.value:
            self.enable_module_outputstream()
        else:
            self.disable_module_outputstream()

    def __toggle_connection_evt(self):
        if not self._connect_btn.checked:
            if hasattr(self, 'arcom'):
                self.disable_stream()
                self._timer.stop()
                self.close()
            self._connect_btn.label = 'Connect'
            self._stream.enabled = False
            self._events.enabled = False
            self._output_stream.enabled = False
            self._zero_btn.enabled = False
            self._reset_threshs.enabled = False
            self._thresh_lower.enabled = False
            self._thresh_upper.enabled = False
            self._start_reading.enabled = False
            self._stream_file.enabled = False

            self._port.enabled = True
            self._refresh_serial_ports.enabled = True
        else:
            try:
                self.open(self._port.value)

                self._connect_btn.label = 'Disconnect'
                self._stream.enabled = True
                self._events.enabled = True
                self._output_stream.enabled = True
                self._zero_btn.enabled = True
                self._reset_threshs.enabled = True
                self._thresh_lower.enabled = True
                self._thresh_upper.enabled = True
                self._start_reading.enabled = True

                self._port.enabled = False
                self._refresh_serial_ports.enabled = False

                if self._filename.value:
                    self._stream_file.enabled = True
                else:
                    self._stream_file.value = False
                    self._stream_file.enabled = False
            except Exception as err:
                self.critical(str(err),  "Error")
                self._connect_btn.checked = False

    def __combo_serial_ports_changed_evt(self):
        self._connect_btn.enabled = True

    def __refresh_serial_ports_btn_pressed(self):
        tmp = self._port.value
        self._port.clear()
        self._fill_serial_ports()
        self._port.value = tmp


if __name__ == '__main__':
    pyforms.start_app(RotaryEncoderModuleGUI, geometry=(0, 0, 600, 500))



# !/usr/bin/python3
# -*- coding: utf-8 -*-


class APIError(Exception):
    """

    """

    def __init__(self, value, original_exception=None):
        self.value = value
        self.original_exception = original_exception

    def __str__(self):
        return self.value



# !/usr/bin/python3
# -*- coding: utf-8 -*-
from confapp import conf
from AnyQt.QtGui import QIcon


from pybpod_rotaryencoder_module.module_gui import RotaryEncoderModuleGUI

class ProjectsRotaryEncoder(object):

	def register_on_main_menu(self, mainmenu):
		super(ProjectsRotaryEncoder, self).register_on_main_menu(mainmenu)

		if len([m for m in mainmenu if 'Tools' in m.keys()]) == 0:
			 mainmenu.append({'Tools': []})

		menu_index = 0
		for i, m in enumerate(mainmenu):
			if 'Tools' in m.keys(): menu_index=i; break

		mainmenu[menu_index]['Tools'].append( '-' )	
		mainmenu[menu_index]['Tools'].append( {'Rotary encoder': self.open_rotaryencoder_plugin, 'icon': QIcon(conf.ROTARYENCODER_PLUGIN_ICON)} )
	
	def open_rotaryencoder_plugin(self):
		if not hasattr(self, 'rotaryencoder_plugin'):
			self.rotaryencoder_plugin = RotaryEncoderModuleGUI(self)
			self.rotaryencoder_plugin.show()
			self.rotaryencoder_plugin.resize(*conf.ROTARYENCODER_PLUGIN_WINDOW_SIZE)			
		else:
			self.rotaryencoder_plugin.show()

		return self.rotaryencoder_plugin


# !/usr/bin/python3
# -*- coding: utf-8 -*-

from pybpodapi.com.messaging.base_message import BaseMessage


class WarningMessage(BaseMessage):

    """ Message that represents an error """

    MESSAGE_TYPE_ALIAS = "warning"
    MESSAGE_COLOR = (255, 100, 0)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from AnyQt.QtGui import QIcon
from confapp import conf
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlButton
from pyforms.controls import ControlCheckBox
from pyforms.controls import ControlTextArea

logger = logging.getLogger(__name__)


class LogWindow(BaseWidget):
    def __init__(self, board):
        BaseWidget.__init__(self, board.name)
        self.board = board
        self.layout().setContentsMargins(5, 5, 5, 5)
        self.setWindowIcon(QIcon(conf.BOARD_SMALL_ICON))
        self._autoscroll_checkbox = ControlCheckBox('Auto-scroll', default=True, changed_event=self.__auto_scroll_evt)
        self._clear_btn = ControlButton('Clear', default=self.__clear_log_evt)
        self._log = ControlTextArea(readonly=True, autoscroll=False)

        self.formset = [(' ', '_autoscroll_checkbox', '_clear_btn'), '_log']

    def __add__(self, data):
        self._log += data
        return self

    def show(self):
        # Prevent the call to be recursive because of the mdi_area
        if hasattr(self, '_show_called'):
            BaseWidget.show(self)
            return
        self._show_called = True
        self.mainwindow.mdi_area += self
        del self._show_called

    def beforeClose(self):
        return False

    def __auto_scroll_evt(self):
        self._log.autoscroll = self._autoscroll_checkbox.value

    def __clear_log_evt(self):
        self._log.value = ''

    @property
    def title(self):
        return BaseWidget.title.fget(self)

    @title.setter
    def title(self, value):
        BaseWidget.title.fset(self, "{0} log".format(value))

    @property
    def mainwindow(self):
        return self.board.mainwindow



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import pyforms
import os
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlTreeView
from pyforms.controls import ControlButton
from pyforms.controls import ControlText
from pyforms.controls import ControlCombo
from pathlib import Path

from AnyQt.QtWidgets import QFileSystemModel

from pybpodgui_api.models.task.taskcommand import TaskCommand


class CommandEditor(BaseWidget):

    def __init__(self, task=None, when=None, command=None):
        title = "Post command editor" if when == TaskCommand.WHEN_POST else "Pre command editor"
        BaseWidget.__init__(self, title, parent_win=task)
        self.command = command
        self.task = task
        self.when = when
        self.set_margin(5)

        self._type = ControlCombo('Type of command', changed_event=self.__type_changed_evt)
        self._cancelbtn = ControlButton('Cancel', default=self.__cancel_evt)
        self._okbtn = ControlButton('Ok', default=self.__ok_evt)

        self._command = ControlText('External command', visible=False)
        self._filesbrowser = ControlTreeView('Files browser')

        self._type.add_item('Execute a gui script', 'script')
        self._type.add_item('Execute a external command', 'external')

        self.formset = [
            '_type',
            '_command',
            '_filesbrowser',
            (' ', '_cancelbtn', '_okbtn'),
            ' '
        ]

        root_path = os.path.abspath(task.path if task else '/')
        self.syspath_model = QFileSystemModel(self)
        self.syspath_model.setRootPath(root_path)
        self.syspath_model.setNameFilters(['*.py'])
        self.syspath_model.setNameFilterDisables(False)
        self._filesbrowser.value = self.syspath_model

        root_index = self.syspath_model.index(root_path)
        self._filesbrowser.setRootIndex(root_index)
        for i in range(1, 4):
            self._filesbrowser.hideColumn(i)

    def __type_changed_evt(self):
        if self._type.value == 'script':
            self._filesbrowser.show()
            self._command.hide()
        elif self._type.value == 'external':
            self._filesbrowser.hide()
            self._command.show()

    def __cancel_evt(self):
        self.close()
        self.task.cmdwin = None

    def __ok_evt(self):

        if self.command is not None:
            command = self.command
        else:
            command = self.task.create_scriptcmd() if self._type.value == 'script' else self.task.create_execcmd()
            command.when = self.when

        if self._type.value == 'script':
            for index in self._filesbrowser.selectedIndexes():
                filepath = self.syspath_model.filePath(index)
                break
            if filepath is None:
                return

            file_path = Path(filepath)
            task_path = Path(self.task.path)
            relpath = file_path.relative_to(task_path)
            command.script = str(relpath)

        elif self._type.value == 'external':
            command.cmd = self._command.value

        self.task.update_commands()
        self.close()
        self.task.cmdwin = None


if __name__ == "__main__":
    pyforms.start_app(CommandEditor)



import pyforms, numpy as np, traceback, math
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlLabel
from pyforms.controls import ControlText
from pyforms.controls import ControlButton
from pyforms.controls import ControlCombo
from pyforms.controls import ControlCheckBox
from pyforms.controls import ControlFile
from pyforms.controls import ControlList
from pyforms.controls import ControlTextArea
from pyforms.controls import ControlNumber
from pyforms.controls import ControlMatplotlib


from .module_api import WavePlayerModule


class OutputChannelGUI(BaseWidget):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.api = kwargs.get('api', None)
        self.channel_index = kwargs.get('channel_index', None)
    
        self.set_margin(10)
        self.setMinimumHeight(60)

        self._evt_report    = ControlCheckBox('Event report', changed_event=self.__event_report_changed_event, default=kwargs.get('event_report',False) )
        self._loop_mode     = ControlCheckBox('Loop mode', changed_event=self.__loop_mode_changed_event, default=kwargs.get('loop_mode',False) )
        self._loop_duration = ControlText('Loop duration', changed_event=self.__loop_duration_changed_event, default=kwargs.get('loop_duration',False) )

        self.formset = [('_evt_report', ' ', '_loop_duration', '_loop_mode')]

    def __event_report_changed_event(self):
        value = self._evt_report.value
        channel_index = self.channel_index-1

        data = self.api.bpod_events
        data[channel_index] = value
        if not self.api.set_bpod_events(data):
            self.alert( 'Not able to set bpod events: {0}'.format(data) )


    def __loop_mode_changed_event(self):
        value = self._loop_mode.value
        channel_index = self.channel_index-1

        data = self.api.loop_mode
        data[channel_index] = value
        try:
            if not self.api.set_loop_mode(data):
                self.alert( 'Not able to set loop mode: {0}'.format(data) )
        except Exception:
            self.critical( traceback.format_exc(), 'An error occurred')


    def __loop_duration_changed_event(self):
        value = self._loop_duration.value
        channel_index = self.channel_index-1

        data = self.api.loop_duration
        data[channel_index] = value
        if not self.api.set_loop_duration(data):
            self.alert( 'Not able to set loop duration: {0}'.format(data) )






class WavePlayerModuleGUI(WavePlayerModule, BaseWidget):

    TITLE = 'Analog Output Module'

    def __init__(self, parent_win = None):
        BaseWidget.__init__(self, self.TITLE, parent_win = parent_win)
        WavePlayerModule.__init__(self)

        self.set_margin(10)

        self._port 			= ControlText('Serial port', default = '/dev/ttyACM0')
        self._connect_btn   = ControlButton('Connect', checkable=True, default = self.__connect_btn_evt)
        self._getparams_btn = ControlButton('Get Parameters', default=self.get_parameters, enabled=False)
        self._info          = ControlTextArea('Information', enabled=False)
        self._channels      = ControlList('Channels', readonly=True, enabled=False)


        self._triggermode = ControlCombo('Channel Select', changed_event=self.__set_trigger_mode_evt, enabled=False)
        self._outputrange = ControlCombo('Output range', changed_event=self.__set_output_range_evt, enabled=False)
        
        self._triggermode.add_item('Normal',0)
        self._triggermode.add_item('Master',1)
        self._triggermode.add_item('Toggle',2)

        self._outputrange.add_item('0V to +5V',0)
        self._outputrange.add_item('0V to +10V',1)
        self._outputrange.add_item('0V to +12V',2)
        self._outputrange.add_item('-5V to +5V',3)
        self._outputrange.add_item('-10V to +10V',4)
        self._outputrange.add_item('-12V to +12V',5)
        
        self._wavegraph  = ControlMatplotlib('Waveform', on_draw=self.__on_draw_evt)
        self._amplitude  = ControlNumber('Amplitude',   default=1.0, minimum=0, maximum=1000, changed_event=self._wavegraph.draw, enabled=False)
        self._duration   = ControlNumber('Duration',    default=3.0, minimum=0, maximum=100, changed_event=self._wavegraph.draw, enabled=False)
        self._frequency  = ControlNumber('Frequency',   default=1000.0, minimum=1, maximum=10000, changed_event=self._wavegraph.draw, enabled=False)
        self._samplerate = ControlNumber('Sample rate', default=96000,  minimum=1, maximum=100000, changed_event=self._wavegraph.draw, enabled=False)
        
        self._wave_index        = ControlNumber('Waveform', enabled=False)
        self._channel_index     = ControlNumber('Channel', minimum=1, default=1, enabled=False)
        self._load_waveform_btn = ControlButton('Load waveform', default=self.__load_waveform_btn_evt, enabled=False)
        self._play_waveform_btn = ControlButton('Play', default=self.__play_btn_evt, enabled=False)
        self._stop_waveform_btn = ControlButton('Stop', default=self.__stop_btn_evt, enabled=False)

        self.formset = [
            {
                'a:Connection' :[
                    ('_port','_connect_btn'), 
                    ('_triggermode', '_outputrange'),
                    ('_amplitude', '_duration', '_frequency', '_samplerate'),
                    '_wavegraph',
                    ('_wave_index','_load_waveform_btn', '_channel_index', '_play_waveform_btn', '_stop_waveform_btn')
                ],
                'c: Channels': ['_channels'],
                'd:Information': ['_getparams_btn','_info']
            }
        ]



    ##########################################################################
    ## EVENTS ################################################################
    ##########################################################################

    def __set_output_range_evt(self):
        if not self.form_has_loaded: return
        self.set_output_range(self._outputrange.value)

    def __set_trigger_mode_evt(self):
        if not self.form_has_loaded: return
        self.set_trigger_mode(self._triggermode.value)

    def __set_trigger_profiles_evt(self):
        self.set_trigger_profiles(self._triggerprofiles.value)



    def __play_btn_evt(self):
        self.play( int(self._channel_index.value), int(self._wave_index.value) )

    def __stop_btn_evt(self):
        self.stop()

    def __on_draw_evt(self, figure):
        try:
            axes = figure.add_subplot(111)
            axes.clear()

            samples = np.arange(0.0, self._duration.value, 1.0/self._samplerate.value)
            wave    = self._amplitude.value * np.sin(2.0*math.pi*self._frequency.value*samples)

            axes.plot(wave)

            y = self._amplitude.value
            x = math.asin( y/self._amplitude.value )/(2.0*math.pi*self._frequency.value)
            
            axes.set_xlim(0, x/(1/self._samplerate.value)*8 )
            axes.set_ylim( np.min(wave), np.max(wave) )
    
            self._wavegraph.repaint()
        except:
            self.critical( traceback.format_exc(), 'An error occurred')

    def __connect_btn_evt(self):
        if self._connect_btn.checked:
            self.open(self._port.value)
        else:
            self.disconnect()

    def __load_waveform_btn_evt(self):
        samples = np.arange(0.0, self._duration.value, 1.0/self._samplerate.value)
        wave    = self._amplitude.value * np.sin(2.0*math.pi*self._frequency.value*samples)
        self.set_sampling_period(self._samplerate.value)
        res = self.load_waveform( int(self._wave_index.value) , wave)
        if not res:
            self.alert('Failed to load the waveform')
    

    ##########################################################################
    ## OVERRIDE FUNCTIONS ####################################################
    ##########################################################################

    def open(self, port):
        res = super().open(port)
        if res:
            self._connect_btn.label = 'Disconnect'
            
            self._channels.enabled = True
            self._getparams_btn.enabled = True
            self._info.enabled = True
            self._channels.enabled = True

            self._outputrange.enabled = True
            self._triggermode.enabled = True

            self._wavegraph.enabled = True
            self._amplitude.enabled = True
            self._duration.enabled = True
            self._frequency.enabled = True
            self._samplerate.enabled = True
            
            self._wave_index.enabled = True
            self._channel_index.enabled = True
            self._load_waveform_btn.enabled = True
            self._play_waveform_btn.enabled = True
            self._stop_waveform_btn.enabled = True

            for i in range(self.n_channels):
                self._channels += ('({0})'.format(i+1), 
                    OutputChannelGUI(
                        api=self,
                        channel_index=i+1,
                        event_report=bool(self.bpod_events[i]),
                        loop_mode=bool(self.loop_mode[i]),
                        loop_duration=str(self.loop_duration[i]),
                    )
                )

            self._wavegraph.draw()
        else:
            self._connect_btn.checked = False
            

    def disconnect(self):
        super().disconnect()

        self._connect_btn.label = 'Connect'

        self._channels.clear()
        self._info.value = ''

        self._channels.enabled = False
        self._getparams_btn.enabled = False
        self._info.enabled = False
        self._channels.enabled = False

        self._outputrange.enabled = False
        self._triggermode.enabled = False

        self._wavegraph.enabled = False
        self._amplitude.enabled = False
        self._duration.enabled = False
        self._frequency.enabled = False
        self._samplerate.enabled = False
        
        self._wave_index.enabled = False
        self._channel_index.enabled = False
        self._load_waveform_btn.enabled = False
        self._play_waveform_btn.enabled = False
        self._stop_waveform_btn.enabled = False


    def get_parameters(self):
        super().get_parameters()

        self._wave_index.max    = self.max_waves-1
        self._channel_index.max = self.n_channels

        text  = ''
        text += 'Number of channels: {0}\n'.format(self.n_channels)
        text += 'Max waves: {0}\n'.format(self.max_waves)
        text += 'Trigger mode: {0}\n'.format(self.trigger_mode)
        text += 'Trigger profile enabled: {0}\n'.format(self.trigger_profile_enable)
        text += 'Number of trigger profiles: {0}\n'.format(self.n_trigger_profiles)
        text += 'Output range: {0}\n'.format(self.output_range)
        text += 'Bpod events: {0}\n'.format(self.bpod_events)
        text += 'Loop mode {0}\n'.format(self.loop_mode)
        text += 'Sampling rates: {0}\n'.format(self.sampling_rate)
        text += 'Loop durations: {0}\n'.format(self.loop_duration)

        self._info.value = text

        self._outputrange.value = self.output_range
        self._triggermode.value = self.trigger_mode


        windows = self._channels.value
        for i in range( len(windows) ):
            windows[i][1]._evt_report.value = self.bpod_events[i]
            windows[i][1]._loop_mode.value = self.loop_mode[i]
            windows[i][1]._loop_duration.value = str(self.loop_duration[i])

    def _stop(self):
        self.stop()

    def before_close_event(self):
        self.close()
        super().before_close_event()



if __name__=='__main__':
    pyforms.start_app( WavePlayerModuleGUI, geometry=(2000,0,600,500) )


# # !/usr/bin/python3
# # -*- coding: utf-8 -*-

import logging

# THESE SETTINGS ARE NEEDED FOR PYSETTINGS
APP_LOG_FILENAME = 'app.log'
APP_LOG_HANDLER_CONSOLE_LEVEL 	= logging.WARNING
APP_LOG_HANDLER_FILE_LEVEL 		= logging.WARNING

# BPOD GUI PLUGIN SETTINGS
APP_LOG_HANDLER_FILE_LEVEL 		= logging.WARNING
APP_LOG_HANDLER_CONSOLE_LEVEL 	= logging.WARNING


# [project], [experiment], [setup], [protocol], [datetime], [subjects]
PYBPODGUI_API_DEFAULT_SESSION_NAME = "[datetime]"


PYBPODGUI_API_CHECK_SUBJECTS_ON_RUN = True

PYBPODGUI_API_AUTO_SAVE_PROJECT_ON_RUN = False



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from pybpodgui_plugin.models.project.project_treenode import ProjectTreeNode

logger = logging.getLogger(__name__)


class ProjectDockWindow(ProjectTreeNode):
    def show(self):
        self.mainwindow.details.value = self
        super(ProjectDockWindow, self).show()

    def focus_name(self):
        """
        Sets interface focus on the board name text field
        """
        self._name.form.lineEdit.setFocus()

    def close(self, silent=False):
        self.mainwindow.details.value = None
        super(ProjectDockWindow, self).close(silent)

    @property
    def mainwindow(self):
        return self.projects.mainwindow



# !/usr/bin/python3
# -*- coding: utf-8 -*-
from confapp import conf
import loggingbootstrap

__version__ = "1.8.2"
__author__ = [
    "Ricardo Ribeiro",
    "Carlos Mão de Ferro",
    "Joshua Sanders",
    "Luís Teixeira",
]
__credits__ = ["Ricardo Ribeiro", "Carlos Mao de Ferro", "Luís Teixeira"]
__license__ = "MIT"
__maintainer__ = [
    "Ricardo Ribeiro",
    "Carlos Mão de Ferro",
    "Joshua Sanders",
    "Luís Teixeira",
]
__email__ = [
    "ricardojvr@gmail.com",
    "cajomferro@gmail.com",
    "joshua21@gmail.com",
    "micboucinha@gmail.com",
]
__status__ = "Development"


# load the user settings
try:
    import user_settings

    conf += user_settings
except:
    pass

conf += "pybpodapi.settings"

if conf.PYBPOD_API_LOG_LEVEL is not None:
    # setup different loggers for example script and api
    loggingbootstrap.create_double_logger(
        "pybpodapi",
        conf.PYBPOD_API_LOG_LEVEL,
        conf.PYBPOD_API_LOG_FILE,
        conf.PYBPOD_API_LOG_LEVEL,
    )



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from confapp import conf

from AnyQt.QtGui import QIcon

from pybpodgui_plugin.models.board.board_dockwindow import BoardDockWindow

logger = logging.getLogger(__name__)


class BoardUIBusy(BoardDockWindow):
    """
    Perform operations related with UI reloading
    """

    def __init__(self, project):
        super(BoardUIBusy, self).__init__(project)

    def update_ui(self):
        """
        update ui now
        """
        if not hasattr(self, 'node'):
            return

        logger.debug('Board [{0}] status: {1}'.format(self.name, self.status))

        if self.status > BoardUIBusy.STATUS_READY:
            self.node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
        else:
            self.node.setIcon(0, QIcon(conf.BOARD_SMALL_ICON))

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def status(self):
        return BoardDockWindow.status.fget(self)

    @status.setter
    def status(self, value):
        BoardDockWindow.status.fset(self, value)
        self.project.update_ui()

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################



import logging

from confapp import conf

from AnyQt.QtGui import QIcon
from pybpodgui_plugin_session_history.session_history import SessionHistory

logger = logging.getLogger(__name__)


class SubjectTreeNode(object):

    def create_sessiontreenode(self, session):
        """
        :param session: 
        :return: 
        """
        node = super(SubjectTreeNode, self).create_sessiontreenode(session)

        self.sessionhistory_action = self.tree.add_popup_menu_option(
            'History', 
            session.open_sessionhistory_win,
            item=node,
            icon=QIcon(conf.SESSIONLOG_PLUGIN_ICON)
        )

        self.sessionhistory_detached_action = self.tree.add_popup_menu_option(
            'History (detached)', 
            session.open_sessionhistory_win_detached,
            item=node,
            icon=QIcon(conf.SESSIONLOG_PLUGIN_ICON)
        )

        return node

    def node_double_clicked_event(self):
        super(SubjectTreeNode, self).node_double_clicked_event()

        self.open_sessionhistory_win()

    def remove(self, silent=False):
        if hasattr(self, 'sessionhistory_win'): self.mainwindow.mdi_area -= self.sessionhistory_win
        super(SubjectTreeNode, self).remove(silent)

    @property
    def name(self):
        return super(SubjectTreeNode, self.__class__).name.fget(self)

    @name.setter
    def name(self, value):
        super(SubjectTreeNode, self.__class__).name.fset(self, value)
        if hasattr(self, 'sessionhistory_win'): self.sessionhistory_win.title = value

